@@ -15,7 +15,7 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA.
  *
  */
 
@@ -23,14 +23,14 @@
 #define __RB_DAAP_SRC_H
 
 #include <glib.h>
+#include <gst/gstelement.h>
 
 G_BEGIN_DECLS
 
-void rb_daap_src_init (void);
-void rb_daap_src_shutdown (void);
+GType rb_daap_src_get_type (void);
 
-void rb_daap_src_set_time (glong time);
-glong rb_daap_src_get_time (void);
+void rb_daap_src_set_time (GstElement *element, glong time);
+glong rb_daap_src_get_time (GstElement *element);
 
 G_END_DECLS
 
