@@ -15,7 +15,7 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA.
  *
  */
 
@@ -83,8 +83,8 @@ rb_dot_dir (void)
 					    GNOME_DOT_GNOME,
 					    "rhythmbox",
 					    NULL);
-		if (!g_file_test (dot_dir, G_FILE_TEST_EXISTS | G_FILE_TEST_IS_DIR))
-			mkdir (dot_dir, 0750);
+		if (mkdir (dot_dir, 0750) == -1)
+			rb_debug ("unable to create Rhythmbox's dot dir");
 	}
 	
 	return dot_dir;
@@ -604,66 +604,97 @@ rb_uri_is_writable (const char *text_uri)
 	return ret;
 }
 
+gboolean
+rb_uri_is_local (const char *text_uri)
+{
+	return g_str_has_prefix (text_uri, "file://");
+}
+
+/*
+ * gnome_vfs_uri_new escapes a few extra characters that
+ * gnome_vfs_escape_path doesn't ('&' and '=').  If we
+ * don't adjust our URIs to match, we end up with duplicate
+ * entries, one with the characters encoded and one without.
+ */
+static char *
+escape_extra_gnome_vfs_chars (char *uri)
+{
+	if (strspn (uri, "&=") != strlen (uri)) {
+		char *tmp = gnome_vfs_escape_set (uri, "&=");
+		g_free (uri);
+		return tmp;
+	}
+
+	return uri;
+}
+
+typedef struct {
+	const char *uri;
+	GFunc func;
+	gpointer user_data;
+	gboolean *cancel_flag;
+} RBUriHandleRecursivelyData;
+
+static gboolean
+rb_uri_handle_recursively_cb (const gchar *rel_path,
+			      GnomeVFSFileInfo *info,
+			      gboolean recursing_will_loop,
+			      RBUriHandleRecursivelyData *data,
+			      gboolean *recurse)
+{
+	if (data->cancel_flag && *data->cancel_flag)
+		return TRUE;
+
+	/* skip hidden and unreadable files and directories */
+	if (g_str_has_prefix (rel_path, ".") ||
+	    ((info->valid_fields & GNOME_VFS_FILE_INFO_FIELDS_ACCESS) &&
+	    !(info->permissions & GNOME_VFS_PERM_ACCESS_READABLE))) {
+		*recurse = FALSE;
+		return TRUE;
+	}
+
+	if (info->type == GNOME_VFS_FILE_TYPE_REGULAR) {
+		char *path, *escaped_rel_path;
+
+		escaped_rel_path = gnome_vfs_escape_path_string (rel_path);
+		escaped_rel_path = escape_extra_gnome_vfs_chars (escaped_rel_path);
+		path = g_build_filename (data->uri, escaped_rel_path, NULL);
+		(data->func) (path, data->user_data);
+		g_free (escaped_rel_path);
+		g_free (path);
+	}
+
+	*recurse = !recursing_will_loop;
+	return TRUE;
+}
+
 void
 rb_uri_handle_recursively (const char *text_uri,
 		           GFunc func,
 			   gboolean *cancelflag,
 		           gpointer user_data)
 {
-	GList *list, *l;
-	GnomeVFSURI *uri;
+	RBUriHandleRecursivelyData *data = g_new0 (RBUriHandleRecursivelyData, 1);
+	GnomeVFSFileInfoOptions flags;
+	GnomeVFSResult result;
+	
+	data->uri = text_uri;
+	data->func = func;
+	data->user_data = user_data;
+	data->cancel_flag = cancelflag;
 
-	uri = gnome_vfs_uri_new (text_uri);
-
-	if (gnome_vfs_directory_list_load (&list, text_uri,
-				           (GNOME_VFS_FILE_INFO_GET_MIME_TYPE |
-					    GNOME_VFS_FILE_INFO_FORCE_FAST_MIME_TYPE |
-					    GNOME_VFS_FILE_INFO_FOLLOW_LINKS)) != GNOME_VFS_OK)
-		return;
-
-	for (l = list; l != NULL; l = g_list_next (l))
-	{
-		GnomeVFSFileInfo *info;
-		GnomeVFSURI *file_uri;
-		char *file_uri_text;
-
-		if (cancelflag && *cancelflag)
-			break;
-
-		info = (GnomeVFSFileInfo *) l->data;
-		
-		file_uri = gnome_vfs_uri_append_path (uri, info->name);
-		if (file_uri == NULL)
-			continue;
-		file_uri_text = gnome_vfs_uri_to_string (file_uri,
-							 GNOME_VFS_URI_HIDE_NONE);
-		gnome_vfs_uri_unref (file_uri);
-
-		if (info->type != GNOME_VFS_FILE_TYPE_REGULAR)
-		{
-			if ((info->type == GNOME_VFS_FILE_TYPE_DIRECTORY) &&
-			    (info->name) && (strcmp (info->name, "."))
-			    && (strcmp (info->name, ".."))
-			    && (strncmp (info->name, ".Trash", strlen (".Trash"))))
-			{
-				rb_uri_handle_recursively (file_uri_text,
-							   func,
-							   cancelflag,
-							   user_data);
-
-			}
-		}
-		else
-			(*func) (file_uri_text, user_data);
-
-		g_free (file_uri_text);
-	}
-
-	gnome_vfs_file_info_list_free (list);
-	gnome_vfs_uri_unref (uri);
+	flags = GNOME_VFS_FILE_INFO_GET_MIME_TYPE |
+		GNOME_VFS_FILE_INFO_FORCE_FAST_MIME_TYPE |
+		GNOME_VFS_FILE_INFO_FOLLOW_LINKS |
+		GNOME_VFS_FILE_INFO_GET_ACCESS_RIGHTS;
+	result = gnome_vfs_directory_visit (text_uri,
+					    flags,
+					    GNOME_VFS_DIRECTORY_VISIT_LOOPCHECK,
+					    (GnomeVFSDirectoryVisitFunc)rb_uri_handle_recursively_cb,
+					    data);
+	g_free (data);
 }
 
-
 GnomeVFSResult
 rb_uri_mkstemp (const char *prefix, char **uri_ret, GnomeVFSHandle **ret)
 {
@@ -699,6 +730,7 @@ rb_canonicalise_uri (const char *uri)
 		result = gnome_vfs_make_path_name_canonical (uri);
 		tmp = gnome_vfs_escape_path_string (result);
 		g_free (result);
+		tmp = escape_extra_gnome_vfs_chars (tmp);
 		result = g_strconcat ("file://", tmp, NULL);
 		g_free (tmp);
 	} else  if (g_str_has_prefix (uri, "file://")) {
@@ -707,6 +739,7 @@ rb_canonicalise_uri (const char *uri)
 		tmp1  = gnome_vfs_unescape_string (uri + 7, NULL);  /* ignore "file://" */
 		tmp2 = gnome_vfs_escape_path_string (tmp1);
 		g_free (tmp1);
+		tmp2 = escape_extra_gnome_vfs_chars (tmp2);
 		result = g_strconcat ("file://", tmp2, NULL); /* re-add scheme */
 		g_free (tmp2);
 	} else {
@@ -717,10 +750,56 @@ rb_canonicalise_uri (const char *uri)
 			gnome_vfs_uri_unref (vfsuri);
 			result = g_strdup (uri);
 		} else {
+			/* this may just mean that gnome-vfs doesn't recognise the
+			 * uri scheme, so return it as is */
 			rb_debug ("Error processing probable URI %s", uri);
+			result = g_strdup (uri);
 		}
 	}
 
 	return result;
 }
 
+char*
+rb_uri_append_path (const char *uri, const char *path)
+{
+	GnomeVFSURI *vfs_uri, *full_uri;
+	char *result;
+	
+	vfs_uri = gnome_vfs_uri_new (uri);
+	full_uri = gnome_vfs_uri_append_path (vfs_uri, path);
+	gnome_vfs_uri_unref (vfs_uri);
+	result = gnome_vfs_uri_to_string (full_uri, GNOME_VFS_URI_HIDE_NONE);
+	gnome_vfs_uri_unref (full_uri);
+
+	return result;
+}
+
+char*
+rb_uri_append_uri (const char *uri, const char *fragment)
+{
+	GnomeVFSURI *vfs_uri, *full_uri;
+	char *result;
+	
+	vfs_uri = gnome_vfs_uri_new (uri);
+	full_uri = gnome_vfs_uri_append_string (vfs_uri, fragment);
+	gnome_vfs_uri_unref (vfs_uri);
+	result = gnome_vfs_uri_to_string (full_uri, GNOME_VFS_URI_HIDE_NONE);
+	gnome_vfs_uri_unref (full_uri);
+
+	return result;
+}
+
+char *
+rb_uri_get_dir_name (const char *uri)
+{
+	GnomeVFSURI *vfs_uri;
+	char *dirname;
+
+	vfs_uri = gnome_vfs_uri_new (uri);
+	dirname = gnome_vfs_uri_extract_dirname (vfs_uri);
+	gnome_vfs_uri_unref (vfs_uri);
+
+	return dirname;
+}
+
