@@ -15,7 +15,7 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA.
  *
  */
 
@@ -33,6 +33,9 @@ GladeXML *rb_glade_xml_new (const char *file,
 void rb_glade_boldify_label (GladeXML *xml,
 			     const char *name);
 
+gboolean rb_combo_box_hyphen_separator_func (GtkTreeModel *model,
+					     GtkTreeIter *iter,
+					     gpointer data);
 G_END_DECLS
 
 #endif /* __RB_GLADE_HELPERS_H */
