@@ -30,7 +30,15 @@ extern void rb_marshal_INT__VOID (GClosure     *closure,
                                   gpointer      invocation_hint,
                                   gpointer      marshal_data);
 
-/* STRING:STRING (./rb-marshal.list:4) */
+/* OBJECT:OBJECT (./rb-marshal.list:4) */
+extern void rb_marshal_OBJECT__OBJECT (GClosure     *closure,
+                                       GValue       *return_value,
+                                       guint         n_param_values,
+                                       const GValue *param_values,
+                                       gpointer      invocation_hint,
+                                       gpointer      marshal_data);
+
+/* STRING:STRING (./rb-marshal.list:5) */
 extern void rb_marshal_STRING__STRING (GClosure     *closure,
                                        GValue       *return_value,
                                        guint         n_param_values,
@@ -38,7 +46,7 @@ extern void rb_marshal_STRING__STRING (GClosure     *closure,
                                        gpointer      invocation_hint,
                                        gpointer      marshal_data);
 
-/* VOID:BOOLEAN,BOOLEAN (./rb-marshal.list:5) */
+/* VOID:BOOLEAN,BOOLEAN (./rb-marshal.list:6) */
 extern void rb_marshal_VOID__BOOLEAN_BOOLEAN (GClosure     *closure,
                                               GValue       *return_value,
                                               guint         n_param_values,
@@ -46,7 +54,31 @@ extern void rb_marshal_VOID__BOOLEAN_BOOLEAN (GClosure     *closure,
                                               gpointer      invocation_hint,
                                               gpointer      marshal_data);
 
-/* VOID:DOUBLE,LONG (./rb-marshal.list:6) */
+/* VOID:BOXED,INT,POINTER,POINTER (./rb-marshal.list:7) */
+extern void rb_marshal_VOID__BOXED_INT_POINTER_POINTER (GClosure     *closure,
+                                                        GValue       *return_value,
+                                                        guint         n_param_values,
+                                                        const GValue *param_values,
+                                                        gpointer      invocation_hint,
+                                                        gpointer      marshal_data);
+
+/* VOID:BOXED,POINTER (./rb-marshal.list:8) */
+extern void rb_marshal_VOID__BOXED_POINTER (GClosure     *closure,
+                                            GValue       *return_value,
+                                            guint         n_param_values,
+                                            const GValue *param_values,
+                                            gpointer      invocation_hint,
+                                            gpointer      marshal_data);
+
+/* VOID:BOXED,ULONG (./rb-marshal.list:9) */
+extern void rb_marshal_VOID__BOXED_ULONG (GClosure     *closure,
+                                          GValue       *return_value,
+                                          guint         n_param_values,
+                                          const GValue *param_values,
+                                          gpointer      invocation_hint,
+                                          gpointer      marshal_data);
+
+/* VOID:DOUBLE,LONG (./rb-marshal.list:10) */
 extern void rb_marshal_VOID__DOUBLE_LONG (GClosure     *closure,
                                           GValue       *return_value,
                                           guint         n_param_values,
@@ -54,7 +86,7 @@ extern void rb_marshal_VOID__DOUBLE_LONG (GClosure     *closure,
                                           gpointer      invocation_hint,
                                           gpointer      marshal_data);
 
-/* VOID:INT,INT,DOUBLE (./rb-marshal.list:7) */
+/* VOID:INT,INT,DOUBLE (./rb-marshal.list:11) */
 extern void rb_marshal_VOID__INT_INT_DOUBLE (GClosure     *closure,
                                              GValue       *return_value,
                                              guint         n_param_values,
@@ -62,7 +94,7 @@ extern void rb_marshal_VOID__INT_INT_DOUBLE (GClosure     *closure,
                                              gpointer      invocation_hint,
                                              gpointer      marshal_data);
 
-/* VOID:INT,POINTER (./rb-marshal.list:8) */
+/* VOID:INT,POINTER (./rb-marshal.list:12) */
 extern void rb_marshal_VOID__INT_POINTER (GClosure     *closure,
                                           GValue       *return_value,
                                           guint         n_param_values,
@@ -70,7 +102,7 @@ extern void rb_marshal_VOID__INT_POINTER (GClosure     *closure,
                                           gpointer      invocation_hint,
                                           gpointer      marshal_data);
 
-/* VOID:OBJECT,INT,INT (./rb-marshal.list:9) */
+/* VOID:OBJECT,INT,INT (./rb-marshal.list:13) */
 extern void rb_marshal_VOID__OBJECT_INT_INT (GClosure     *closure,
                                              GValue       *return_value,
                                              guint         n_param_values,
@@ -78,7 +110,7 @@ extern void rb_marshal_VOID__OBJECT_INT_INT (GClosure     *closure,
                                              gpointer      invocation_hint,
                                              gpointer      marshal_data);
 
-/* VOID:OBJECT,INT,INT,BOXED,UINT,UINT (./rb-marshal.list:10) */
+/* VOID:OBJECT,INT,INT,BOXED,UINT,UINT (./rb-marshal.list:14) */
 extern void rb_marshal_VOID__OBJECT_INT_INT_BOXED_UINT_UINT (GClosure     *closure,
                                                              GValue       *return_value,
                                                              guint         n_param_values,
@@ -86,7 +118,15 @@ extern void rb_marshal_VOID__OBJECT_INT_INT_BOXED_UINT_UINT (GClosure     *closu
                                                              gpointer      invocation_hint,
                                                              gpointer      marshal_data);
 
-/* VOID:POINTER,INT (./rb-marshal.list:11) */
+/* VOID:OBJECT,INT,POINTER (./rb-marshal.list:15) */
+extern void rb_marshal_VOID__OBJECT_INT_POINTER (GClosure     *closure,
+                                                 GValue       *return_value,
+                                                 guint         n_param_values,
+                                                 const GValue *param_values,
+                                                 gpointer      invocation_hint,
+                                                 gpointer      marshal_data);
+
+/* VOID:POINTER,INT (./rb-marshal.list:16) */
 extern void rb_marshal_VOID__POINTER_INT (GClosure     *closure,
                                           GValue       *return_value,
                                           guint         n_param_values,
@@ -94,7 +134,7 @@ extern void rb_marshal_VOID__POINTER_INT (GClosure     *closure,
                                           gpointer      invocation_hint,
                                           gpointer      marshal_data);
 
-/* VOID:POINTER,INT,POINTER (./rb-marshal.list:12) */
+/* VOID:POINTER,INT,POINTER (./rb-marshal.list:17) */
 extern void rb_marshal_VOID__POINTER_INT_POINTER (GClosure     *closure,
                                                   GValue       *return_value,
                                                   guint         n_param_values,
@@ -102,15 +142,7 @@ extern void rb_marshal_VOID__POINTER_INT_POINTER (GClosure     *closure,
                                                   gpointer      invocation_hint,
                                                   gpointer      marshal_data);
 
-/* VOID:POINTER,INT,POINTER,POINTER (./rb-marshal.list:13) */
-extern void rb_marshal_VOID__POINTER_INT_POINTER_POINTER (GClosure     *closure,
-                                                          GValue       *return_value,
-                                                          guint         n_param_values,
-                                                          const GValue *param_values,
-                                                          gpointer      invocation_hint,
-                                                          gpointer      marshal_data);
-
-/* VOID:POINTER,POINTER (./rb-marshal.list:14) */
+/* VOID:POINTER,POINTER (./rb-marshal.list:18) */
 extern void rb_marshal_VOID__POINTER_POINTER (GClosure     *closure,
                                               GValue       *return_value,
                                               guint         n_param_values,
@@ -118,7 +150,7 @@ extern void rb_marshal_VOID__POINTER_POINTER (GClosure     *closure,
                                               gpointer      invocation_hint,
                                               gpointer      marshal_data);
 
-/* VOID:POINTER,ULONG (./rb-marshal.list:15) */
+/* VOID:POINTER,ULONG (./rb-marshal.list:19) */
 extern void rb_marshal_VOID__POINTER_ULONG (GClosure     *closure,
                                             GValue       *return_value,
                                             guint         n_param_values,
@@ -126,7 +158,7 @@ extern void rb_marshal_VOID__POINTER_ULONG (GClosure     *closure,
                                             gpointer      invocation_hint,
                                             gpointer      marshal_data);
 
-/* VOID:STRING,DOUBLE (./rb-marshal.list:16) */
+/* VOID:STRING,DOUBLE (./rb-marshal.list:20) */
 extern void rb_marshal_VOID__STRING_DOUBLE (GClosure     *closure,
                                             GValue       *return_value,
                                             guint         n_param_values,
@@ -134,7 +166,7 @@ extern void rb_marshal_VOID__STRING_DOUBLE (GClosure     *closure,
                                             gpointer      invocation_hint,
                                             gpointer      marshal_data);
 
-/* VOID:STRING,INT (./rb-marshal.list:17) */
+/* VOID:STRING,INT (./rb-marshal.list:21) */
 extern void rb_marshal_VOID__STRING_INT (GClosure     *closure,
                                          GValue       *return_value,
                                          guint         n_param_values,
@@ -142,7 +174,7 @@ extern void rb_marshal_VOID__STRING_INT (GClosure     *closure,
                                          gpointer      invocation_hint,
                                          gpointer      marshal_data);
 
-/* VOID:STRING,POINTER (./rb-marshal.list:18) */
+/* VOID:STRING,POINTER (./rb-marshal.list:22) */
 extern void rb_marshal_VOID__STRING_POINTER (GClosure     *closure,
                                              GValue       *return_value,
                                              guint         n_param_values,
@@ -150,7 +182,7 @@ extern void rb_marshal_VOID__STRING_POINTER (GClosure     *closure,
                                              gpointer      invocation_hint,
                                              gpointer      marshal_data);
 
-/* VOID:STRING,STRING (./rb-marshal.list:19) */
+/* VOID:STRING,STRING (./rb-marshal.list:23) */
 extern void rb_marshal_VOID__STRING_STRING (GClosure     *closure,
                                             GValue       *return_value,
                                             guint         n_param_values,
@@ -158,7 +190,7 @@ extern void rb_marshal_VOID__STRING_STRING (GClosure     *closure,
                                             gpointer      invocation_hint,
                                             gpointer      marshal_data);
 
-/* VOID:STRING,STRING,STRING (./rb-marshal.list:20) */
+/* VOID:STRING,STRING,STRING (./rb-marshal.list:24) */
 extern void rb_marshal_VOID__STRING_STRING_STRING (GClosure     *closure,
                                                    GValue       *return_value,
                                                    guint         n_param_values,
@@ -166,7 +198,7 @@ extern void rb_marshal_VOID__STRING_STRING_STRING (GClosure     *closure,
                                                    gpointer      invocation_hint,
                                                    gpointer      marshal_data);
 
-/* VOID:STRING,STRING,STRING,UINT,BOOLEAN (./rb-marshal.list:21) */
+/* VOID:STRING,STRING,STRING,UINT,BOOLEAN (./rb-marshal.list:25) */
 extern void rb_marshal_VOID__STRING_STRING_STRING_UINT_BOOLEAN (GClosure     *closure,
                                                                 GValue       *return_value,
                                                                 guint         n_param_values,
@@ -174,6 +206,30 @@ extern void rb_marshal_VOID__STRING_STRING_STRING_UINT_BOOLEAN (GClosure     *cl
                                                                 gpointer      invocation_hint,
                                                                 gpointer      marshal_data);
 
+/* VOID:ULONG,FLOAT (./rb-marshal.list:26) */
+extern void rb_marshal_VOID__ULONG_FLOAT (GClosure     *closure,
+                                          GValue       *return_value,
+                                          guint         n_param_values,
+                                          const GValue *param_values,
+                                          gpointer      invocation_hint,
+                                          gpointer      marshal_data);
+
+/* VOID:OBJECT,BOOLEAN (./rb-marshal.list:27) */
+extern void rb_marshal_VOID__OBJECT_BOOLEAN (GClosure     *closure,
+                                             GValue       *return_value,
+                                             guint         n_param_values,
+                                             const GValue *param_values,
+                                             gpointer      invocation_hint,
+                                             gpointer      marshal_data);
+
+/* VOID:STRING,STRING,POINTER,POINTER (./rb-marshal.list:28) */
+extern void rb_marshal_VOID__STRING_STRING_POINTER_POINTER (GClosure     *closure,
+                                                            GValue       *return_value,
+                                                            guint         n_param_values,
+                                                            const GValue *param_values,
+                                                            gpointer      invocation_hint,
+                                                            gpointer      marshal_data);
+
 G_END_DECLS
 
 #endif /* __rb_marshal_MARSHAL_H__ */
