@@ -1,4 +1,5 @@
-/*
+/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*-
+ *
  * heavily based on code from Gedit
  *
  * Copyright (C) 2002 Paolo Maggi and James Willcox
@@ -16,8 +17,8 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, 
- * Boston, MA 02110-1301  USA. 
+ * Foundation, Inc., 51 Franklin St, Fifth Floor,
+ * Boston, MA 02110-1301  USA.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -28,7 +29,6 @@
 
 #include <glib/gi18n.h>
 #include <glade/glade-xml.h>
-#include <libgnome/gnome-url.h>
 
 #include "rb-plugin-manager.h"
 #include "rb-plugins-engine.h"
@@ -39,7 +39,8 @@
 enum
 {
 	ACTIVE_COLUMN,
-	NAME_COLUMN,
+	VISIBLE_COLUMN,
+	INFO_COLUMN,
 	N_COLUMNS
 };
 
@@ -50,8 +51,9 @@ enum
 
 struct _RBPluginManagerPrivate
 {
-	const GList	*plugins;
+	GList		*plugins;
 	GtkWidget	*tree;
+	GtkTreeModel	*plugin_model;
 
 	GtkWidget	*configure_button;
 	GtkWidget	*site_label;
@@ -68,15 +70,18 @@ struct _RBPluginManagerPrivate
 
 G_DEFINE_TYPE(RBPluginManager, rb_plugin_manager, GTK_TYPE_VBOX)
 
-static RBPluginInfo *plugin_manager_get_selected_plugin (RBPluginManager *pm); 
-static void plugin_manager_toggle_active (GtkTreeIter *iter, GtkTreeModel *model, RBPluginManager *pm); 
-static void plugin_manager_toggle_all (RBPluginManager *pm); 
+static void rb_plugin_manager_finalize (GObject *o);
+static RBPluginInfo *plugin_manager_get_selected_plugin (RBPluginManager *pm);
+static void plugin_manager_toggle_active (GtkTreeIter *iter, GtkTreeModel *model, RBPluginManager *pm);
+static void plugin_manager_toggle_all (RBPluginManager *pm);
 
-static void 
+static void
 rb_plugin_manager_class_init (RBPluginManagerClass *klass)
 {
 	GObjectClass *object_class = G_OBJECT_CLASS (klass);
 
+	object_class->finalize = rb_plugin_manager_finalize;
+
 	g_type_class_add_private (object_class, sizeof (RBPluginManagerPrivate));
 }
 
@@ -97,7 +102,7 @@ configure_button_cb (GtkWidget          *button,
 
 	rb_plugins_engine_configure_plugin (info, toplevel);
 
-	rb_debug ("Done configuring plugin");	
+	rb_debug ("Done configuring plugin");
 }
 
 static void
@@ -108,18 +113,18 @@ plugin_manager_view_cell_cb (GtkTreeViewColumn *tree_column,
 			     gpointer           data)
 {
 	RBPluginInfo *info;
-	
+
 	g_return_if_fail (tree_model != NULL);
 	g_return_if_fail (tree_column != NULL);
 
-	gtk_tree_model_get (tree_model, iter, NAME_COLUMN, &info, -1);
+	gtk_tree_model_get (tree_model, iter, INFO_COLUMN, &info, -1);
 
 	if (info == NULL)
 		return;
 
-	g_object_set (G_OBJECT (cell), 
-		      "text", 
-		      rb_plugins_engine_get_plugin_name (info), 
+	g_object_set (G_OBJECT (cell),
+		      "text",
+		      rb_plugins_engine_get_plugin_name (info),
 		      NULL);
 }
 
@@ -175,7 +180,7 @@ cursor_changed_cb (GtkTreeSelection *selection,
 				   rb_plugins_engine_get_plugin_icon (info));
 
 	gtk_widget_set_sensitive (GTK_WIDGET (pm->priv->configure_button),
-				  (info != NULL) && 
+				  (info != NULL) &&
 				   rb_plugins_engine_plugin_is_configurable (info));
 }
 
@@ -203,9 +208,7 @@ static void
 column_clicked_cb (GtkTreeViewColumn *tree_column,
 		   gpointer           data)
 {
-	RBPluginManager *pm = data;
-
-	g_return_if_fail (pm != NULL);
+	RBPluginManager *pm = RB_PLUGIN_MANAGER (data);
 
 	plugin_manager_toggle_all (pm);
 }
@@ -213,40 +216,32 @@ column_clicked_cb (GtkTreeViewColumn *tree_column,
 static void
 plugin_manager_populate_lists (RBPluginManager *pm)
 {
-	const GList *plugins;
-	GtkListStore *model;
+	GtkTreeModel *model;
 	GtkTreeIter iter;
+	GList *p;
 
-	plugins = pm->priv->plugins;
-
-	model = GTK_LIST_STORE (gtk_tree_view_get_model (GTK_TREE_VIEW (pm->priv->tree)));
-
-	while (plugins)	{
+	for (p = pm->priv->plugins; p != NULL; p = g_list_next (p)) {
 		RBPluginInfo *info;
-		info = (RBPluginInfo *)plugins->data;
+		info = (RBPluginInfo *)p->data;
 
-		gtk_list_store_append (model, &iter);
-		gtk_list_store_set (model, &iter,
+		gtk_list_store_append (GTK_LIST_STORE (pm->priv->plugin_model), &iter);
+		gtk_list_store_set (GTK_LIST_STORE (pm->priv->plugin_model), &iter,
 				    ACTIVE_COLUMN, rb_plugins_engine_plugin_is_active (info),
-				    NAME_COLUMN, info,
+				    VISIBLE_COLUMN, rb_plugins_engine_plugin_is_visible (info),
+				    INFO_COLUMN, info,
 				    -1);
-
-		plugins = plugins->next;
 	}
 
+	model = gtk_tree_view_get_model (GTK_TREE_VIEW (pm->priv->tree));
 	if (gtk_tree_model_get_iter_first (GTK_TREE_MODEL (model), &iter)) {
 		GtkTreeSelection *selection;
-		RBPluginInfo* info;
 
 		selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (pm->priv->tree));
 		g_return_if_fail (selection != NULL);
-		
-		gtk_tree_selection_select_iter (selection, &iter);
 
-		gtk_tree_model_get (GTK_TREE_MODEL (model), &iter,
-				    NAME_COLUMN, &info, -1);
-		/*cursor_changed_cb (selection, pm);*/
+		gtk_tree_selection_select_iter (selection, &iter);
 	}
+	g_object_unref (model);
 }
 
 static void
@@ -256,8 +251,9 @@ plugin_manager_set_active (GtkTreeIter  *iter,
 			   RBPluginManager *pm)
 {
 	RBPluginInfo *info;
-	
-	gtk_tree_model_get (model, iter, NAME_COLUMN, &info, -1);
+	GtkTreeIter child_iter;
+
+	gtk_tree_model_get (model, iter, INFO_COLUMN, &info, -1);
 
 	g_return_if_fail (info != NULL);
 
@@ -274,12 +270,14 @@ plugin_manager_set_active (GtkTreeIter  *iter,
 			active ^= 1;
 		}
 	}
-  
+
 	/* set new value */
-	gtk_list_store_set (GTK_LIST_STORE (model), 
-			    iter, 
+	gtk_tree_model_filter_convert_iter_to_child_iter (GTK_TREE_MODEL_FILTER (model),
+							  &child_iter, iter);
+	gtk_list_store_set (GTK_LIST_STORE (pm->priv->plugin_model),
+			    &child_iter,
 			    ACTIVE_COLUMN,
-			    rb_plugins_engine_plugin_is_active (info), 
+			    rb_plugins_engine_plugin_is_active (info),
 			    -1);
 
 	/* cause the configure button sensitivity to be updated */
@@ -291,12 +289,17 @@ plugin_manager_toggle_active (GtkTreeIter  *iter,
 			      GtkTreeModel *model,
 			      RBPluginManager *pm)
 {
-	gboolean active;
-	
-	gtk_tree_model_get (model, iter, ACTIVE_COLUMN, &active, -1);
+	gboolean active, visible;
 
-	active ^= 1;
-	plugin_manager_set_active (iter, model, active, pm);
+	gtk_tree_model_get (model, iter,
+			    ACTIVE_COLUMN, &active,
+			    VISIBLE_COLUMN, &visible,
+			    -1);
+
+	if (visible) {
+		active ^= 1;
+		plugin_manager_set_active (iter, model, active, pm);
+	}
 }
 
 static RBPluginInfo *
@@ -314,9 +317,9 @@ plugin_manager_get_selected_plugin (RBPluginManager *pm)
 	g_return_val_if_fail (selection != NULL, NULL);
 
 	if (gtk_tree_selection_get_selected (selection, NULL, &iter)) {
-		gtk_tree_model_get (model, &iter, NAME_COLUMN, &info, -1);
+		gtk_tree_model_get (model, &iter, INFO_COLUMN, &info, -1);
 	}
-	
+
 	return info;
 }
 
@@ -356,7 +359,7 @@ name_search_cb (GtkTreeModel *model,
 	gint key_len;
 	gboolean retval;
 
-	gtk_tree_model_get (model, iter, NAME_COLUMN, &info, -1);
+	gtk_tree_model_get (model, iter, INFO_COLUMN, &info, -1);
 	if (!info)
 		return FALSE;
 
@@ -385,12 +388,13 @@ plugin_manager_construct_tree (RBPluginManager *pm)
 {
 	GtkTreeViewColumn *column;
 	GtkCellRenderer *cell;
-	GtkListStore *model;
+	GtkTreeModel *filter;
 
-	model = gtk_list_store_new (N_COLUMNS, G_TYPE_BOOLEAN, G_TYPE_POINTER);
-
-	gtk_tree_view_set_model (GTK_TREE_VIEW (pm->priv->tree), GTK_TREE_MODEL (model));
-	g_object_unref (model);
+	pm->priv->plugin_model = GTK_TREE_MODEL (gtk_list_store_new (N_COLUMNS, G_TYPE_BOOLEAN, G_TYPE_BOOLEAN, G_TYPE_POINTER));
+	filter = gtk_tree_model_filter_new (pm->priv->plugin_model, NULL);
+	gtk_tree_model_filter_set_visible_column (GTK_TREE_MODEL_FILTER (filter), VISIBLE_COLUMN);
+	gtk_tree_view_set_model (GTK_TREE_VIEW (pm->priv->tree), filter);
+	g_object_unref (filter);
 
 	gtk_tree_view_set_rules_hint (GTK_TREE_VIEW (pm->priv->tree), TRUE);
 	gtk_tree_view_set_headers_clickable (GTK_TREE_VIEW (pm->priv->tree), TRUE);
@@ -420,7 +424,7 @@ plugin_manager_construct_tree (RBPluginManager *pm)
 	gtk_tree_view_append_column (GTK_TREE_VIEW (pm->priv->tree), column);
 
 	/* Enable search for our non-string column */
-	gtk_tree_view_set_search_column (GTK_TREE_VIEW (pm->priv->tree), NAME_COLUMN);
+	gtk_tree_view_set_search_column (GTK_TREE_VIEW (pm->priv->tree), INFO_COLUMN);
 	gtk_tree_view_set_search_equal_func (GTK_TREE_VIEW (pm->priv->tree),
 					     name_search_cb,
 					     NULL,
@@ -438,7 +442,7 @@ plugin_manager_construct_tree (RBPluginManager *pm)
 	gtk_widget_show (pm->priv->tree);
 }
 
-static void 
+static void
 rb_plugin_manager_init (RBPluginManager *pm)
 {
 	GladeXML *xml;
@@ -456,7 +460,7 @@ rb_plugin_manager_init (RBPluginManager *pm)
 	pm->priv->tree = gtk_tree_view_new ();
 	plugins_tree_vbox = glade_xml_get_widget (xml, "plugins_tree_vbox");
 	gtk_container_add (GTK_CONTAINER (plugins_tree_vbox), pm->priv->tree);
-	
+
 	pm->priv->configure_button = glade_xml_get_widget (xml, "configure_button");
 	g_signal_connect (pm->priv->configure_button,
 			  "clicked",
@@ -464,7 +468,7 @@ rb_plugin_manager_init (RBPluginManager *pm)
 			  pm);
 
 	pm->priv->plugin_title = glade_xml_get_widget (xml, "plugin_title");
-	
+
 	pm->priv->site_label = glade_xml_get_widget (xml, "site_label");
 	rb_glade_boldify_label (xml, "site_label");
 	pm->priv->copyright_label = glade_xml_get_widget (xml, "copyright_label");
@@ -488,7 +492,16 @@ rb_plugin_manager_init (RBPluginManager *pm)
 	plugin_manager_populate_lists (pm);
 }
 
-GtkWidget *rb_plugin_manager_new (void)
+GtkWidget *
+rb_plugin_manager_new (void)
 {
 	return g_object_new (RB_TYPE_PLUGIN_MANAGER, 0);
 }
+
+static void
+rb_plugin_manager_finalize (GObject *o)
+{
+	RBPluginManager *pm = RB_PLUGIN_MANAGER (o);
+
+	g_list_free (pm->priv->plugins);
+}
