@@ -1,4 +1,5 @@
-/* 
+/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*-
+ *
  *  arch-tag: Implementation of RhythmDB property GtkTreeModel
  *
  *  Copyright (C) 2003 Colin Walters <walters@gnome.org>
@@ -15,19 +16,21 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA.
  *
  */
 
-#include <config.h>
+#include "config.h"
+
 #include <unistd.h>
 #include <stdlib.h>
 #include <string.h>
+#include <glib/gi18n.h>
 
-#include <libgnome/gnome-i18n.h>
 #include "rhythmdb-property-model.h"
 #include "rb-debug.h"
 #include "gsequence.h"
+#include "rb-refstring.h"
 #include "rb-tree-dnd.h"
 
 static void rhythmdb_property_model_tree_model_init (GtkTreeModelIface *iface);
@@ -39,6 +42,12 @@ G_DEFINE_TYPE_WITH_CODE(RhythmDBPropertyModel, rhythmdb_property_model, G_TYPE_O
 			G_IMPLEMENT_INTERFACE(RB_TYPE_TREE_DRAG_SOURCE,
 					      rhythmdb_property_model_drag_source_init))
 
+typedef struct {
+	RBRefString *string;
+	RBRefString *sort_string;
+	guint refcount;
+} RhythmDBPropertyModelEntry;
+
 static void rhythmdb_property_model_finalize (GObject *object);
 static void rhythmdb_property_model_set_property (GObject *object,
 					       guint prop_id,
@@ -48,6 +57,7 @@ static void rhythmdb_property_model_get_property (GObject *object,
 					       guint prop_id,
 					       GValue *value,
 					       GParamSpec *pspec);
+static void rhythmdb_property_model_sync (RhythmDBPropertyModel *model);
 static void rhythmdb_property_model_row_inserted_cb (GtkTreeModel *model,
 						     GtkTreePath *path,
 						     GtkTreeIter *iter,
@@ -59,14 +69,12 @@ static void rhythmdb_property_model_prop_changed_cb (RhythmDB *db, RhythmDBEntry
 static void rhythmdb_property_model_entry_removed_cb (RhythmDBQueryModel *model,
 						      RhythmDBEntry *entry,
 						      RhythmDBPropertyModel *propmodel);
-static void rhythmdb_property_model_insert (RhythmDBPropertyModel *model, RhythmDBEntry *entry);
+static RhythmDBPropertyModelEntry* rhythmdb_property_model_insert (RhythmDBPropertyModel *model,
+								   RhythmDBEntry *entry);
 static void rhythmdb_property_model_delete (RhythmDBPropertyModel *model,
 					    RhythmDBEntry *entry);
-static void rhythmdb_property_model_insert_prop (RhythmDBPropertyModel *model,
-							 const char *propstr,
-							 const char *sort_key);
 static void rhythmdb_property_model_delete_prop (RhythmDBPropertyModel *model,
-							 const char *propstr);
+						 const char *propstr);
 static GtkTreeModelFlags rhythmdb_property_model_get_flags (GtkTreeModel *model);
 static gint rhythmdb_property_model_get_n_columns (GtkTreeModel *tree_model);
 static GType rhythmdb_property_model_get_column_type (GtkTreeModel *tree_model, int index);
@@ -99,10 +107,12 @@ static gboolean rhythmdb_property_model_drag_data_delete (RbTreeDragSource *drag
 							  GList *paths);
 static gboolean rhythmdb_property_model_row_draggable (RbTreeDragSource *dragsource,
 						       GList *paths);
+
 enum {
 	TARGET_ALBUMS,
 	TARGET_GENRE,
 	TARGET_ARTISTS,
+	TARGET_LOCATION,
 	TARGET_URIS,
 };
 
@@ -118,39 +128,38 @@ static const GtkTargetEntry targets_artist [] = {
 	{ "text/x-rhythmbox-artist", 0, TARGET_ARTISTS },
 	{ "text/uri-list", 0, TARGET_URIS },
 };
+static const GtkTargetEntry targets_location [] = {
+	{ "text/x-rhythmbox-location", 0, TARGET_LOCATION },
+	{ "text/uri-list", 0, TARGET_URIS },
+};
 
 static GtkTargetList *rhythmdb_property_model_album_drag_target_list = NULL;
 static GtkTargetList *rhythmdb_property_model_artist_drag_target_list = NULL;
 static GtkTargetList *rhythmdb_property_model_genre_drag_target_list = NULL;
-
-typedef struct {
-	char *name;
-	char *sort_key;
-	guint refcount;
-} RhythmDBPropertyModelEntry;
+static GtkTargetList *rhythmdb_property_model_location_drag_target_list = NULL;
 
 struct RhythmDBPropertyModelPrivate
 {
 	RhythmDB *db;
 
 	RhythmDBQueryModel *query_model;
+	GHashTable *entries;
 
 	RhythmDBPropType propid;
 	RhythmDBPropType sort_propid;
 
 	guint stamp;
 
-	GPtrArray *query;
-
 	GSequence *properties;
-	GMemChunk *property_memchunk;
 	GHashTable *reverse_map;
 
 	RhythmDBPropertyModelEntry *all;
-	
-	gboolean complete;
+
+	guint syncing_id;
 };
 
+#define RHYTHMDB_PROPERTY_MODEL_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), RHYTHMDB_TYPE_PROPERTY_MODEL, RhythmDBPropertyModelPrivate))
+
 enum
 {
 	PRE_ROW_DELETION,
@@ -161,7 +170,6 @@ enum
 {
 	PROP_0,
 	PROP_RHYTHMDB,
-	PROP_QUERY,
 	PROP_PROP,
 	PROP_QUERY_MODEL,
 };
@@ -185,6 +193,10 @@ rhythmdb_property_model_class_init (RhythmDBPropertyModelClass *klass)
 		rhythmdb_property_model_genre_drag_target_list =
 			gtk_target_list_new (targets_genre,
 					     G_N_ELEMENTS (targets_genre));
+	if (!rhythmdb_property_model_location_drag_target_list)
+		rhythmdb_property_model_location_drag_target_list =
+			gtk_target_list_new (targets_location,
+					     G_N_ELEMENTS (targets_location));
 
 	object_class->set_property = rhythmdb_property_model_set_property;
 	object_class->get_property = rhythmdb_property_model_get_property;
@@ -209,13 +221,6 @@ rhythmdb_property_model_class_init (RhythmDBPropertyModelClass *klass)
 							      RHYTHMDB_TYPE,
 							      G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
 
-	g_object_class_install_property (object_class,
-					 PROP_QUERY,
-					 g_param_spec_pointer ("query",
-							      "Query",
-							      "RhythmDBQuery",
-							      G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
-
 	g_object_class_install_property (object_class,
 					 PROP_PROP,
 					 g_param_spec_int ("prop",
@@ -232,6 +237,7 @@ rhythmdb_property_model_class_init (RhythmDBPropertyModelClass *klass)
 							      RHYTHMDB_TYPE_QUERY_MODEL,
 							      G_PARAM_READWRITE));
 
+	g_type_class_add_private (klass, sizeof (RhythmDBPropertyModelPrivate));
 }
 
 static void
@@ -259,41 +265,110 @@ rhythmdb_property_model_drag_source_init (RbTreeDragSourceIface *iface)
 	iface->drag_data_get = rhythmdb_property_model_drag_data_get;
 }
 
+static gboolean
+_remove_entry_cb (GtkTreeModel *model,
+		  GtkTreePath *path,
+		  GtkTreeIter *iter,
+		  RhythmDBPropertyModel *propmodel)
+{
+	RhythmDBEntry *entry;
+
+	entry = rhythmdb_query_model_iter_to_entry (RHYTHMDB_QUERY_MODEL (model), iter);
+	rhythmdb_property_model_entry_removed_cb (RHYTHMDB_QUERY_MODEL (model),
+						  entry,
+						  propmodel);
+	return FALSE;
+}
+
+static gboolean
+_add_entry_cb (GtkTreeModel *model,
+	       GtkTreePath *path,
+	       GtkTreeIter *iter,
+	       RhythmDBPropertyModel *propmodel)
+{
+	rhythmdb_property_model_row_inserted_cb (model, path, iter, propmodel);
+	return FALSE;
+}
+
+static void
+rhythmdb_property_model_set_query_model_internal (RhythmDBPropertyModel *model,
+						  RhythmDBQueryModel    *query_model)
+{
+	if (model->priv->query_model != NULL) {
+		g_signal_handlers_disconnect_by_func (model->priv->query_model,
+						      G_CALLBACK (rhythmdb_property_model_row_inserted_cb),
+						      model);
+		g_signal_handlers_disconnect_by_func (model->priv->query_model,
+						      G_CALLBACK (rhythmdb_property_model_entry_removed_cb),
+						      model);
+		g_signal_handlers_disconnect_by_func (model->priv->query_model,
+						      G_CALLBACK (rhythmdb_property_model_prop_changed_cb),
+						      model);
+
+		gtk_tree_model_foreach (GTK_TREE_MODEL (model->priv->query_model),
+					(GtkTreeModelForeachFunc)_remove_entry_cb,
+					model);
+
+		g_object_unref (model->priv->query_model);
+	}
+
+	model->priv->query_model = query_model;
+	g_assert (rhythmdb_property_model_iter_n_children (GTK_TREE_MODEL (model), NULL) == 1);
+
+	if (model->priv->query_model != NULL) {
+		g_object_ref (model->priv->query_model);
+
+		g_signal_connect_object (model->priv->query_model,
+					 "row_inserted",
+					 G_CALLBACK (rhythmdb_property_model_row_inserted_cb),
+					 model,
+					 0);
+		g_signal_connect_object (model->priv->query_model,
+					 "post-entry-delete",
+					 G_CALLBACK (rhythmdb_property_model_entry_removed_cb),
+					 model,
+					 0);
+		g_signal_connect_object (model->priv->query_model,
+					 "entry-prop-changed",
+					 G_CALLBACK (rhythmdb_property_model_prop_changed_cb),
+					 model,
+					 0);
+		gtk_tree_model_foreach (GTK_TREE_MODEL (model->priv->query_model),
+					(GtkTreeModelForeachFunc)_add_entry_cb,
+					model);
+	}
+}
+
 static void
 rhythmdb_property_model_set_property (GObject *object,
-				   guint prop_id,
-				   const GValue *value,
-				   GParamSpec *pspec)
+				      guint prop_id,
+				      const GValue *value,
+				      GParamSpec *pspec)
 {
 	RhythmDBPropertyModel *model = RHYTHMDB_PROPERTY_MODEL (object);
 
-	switch (prop_id)
-	{
+	switch (prop_id) {
 	case PROP_RHYTHMDB:
-	{
 		model->priv->db = g_value_get_object (value);
 		break;
-	}
-	case PROP_QUERY:
-		if (model->priv->query)
-			rhythmdb_query_free (model->priv->query);
-		model->priv->query = rhythmdb_query_copy (g_value_get_pointer (value));
-		break;
 	case PROP_PROP:
 		model->priv->propid = g_value_get_int (value);
-		switch (model->priv->propid)
-		{
+		switch (model->priv->propid) {
 		case RHYTHMDB_PROP_GENRE:
-			model->priv->sort_propid = RHYTHMDB_PROP_GENRE_SORT_KEY;
+			model->priv->sort_propid = RHYTHMDB_PROP_GENRE;
 			break;
 		case RHYTHMDB_PROP_ARTIST:
-			model->priv->sort_propid = RHYTHMDB_PROP_ARTIST_SORT_KEY;
+			model->priv->sort_propid = RHYTHMDB_PROP_ARTIST;
 			break;
 		case RHYTHMDB_PROP_ALBUM:
-			model->priv->sort_propid = RHYTHMDB_PROP_ALBUM_SORT_KEY;
+			model->priv->sort_propid = RHYTHMDB_PROP_ALBUM;
+			break;
+		case RHYTHMDB_PROP_SUBTITLE:
+			model->priv->sort_propid = RHYTHMDB_PROP_SUBTITLE;
 			break;
 		case RHYTHMDB_PROP_TITLE:
-			model->priv->sort_propid = RHYTHMDB_PROP_TITLE_SORT_KEY;
+		case RHYTHMDB_PROP_LOCATION:
+			model->priv->sort_propid = RHYTHMDB_PROP_TITLE;
 			break;
 		default:
 			g_assert_not_reached ();
@@ -301,64 +376,26 @@ rhythmdb_property_model_set_property (GObject *object,
 		}
 		break;
 	case PROP_QUERY_MODEL:
-	{
-		if (model->priv->query_model) {
-			g_signal_handlers_disconnect_by_func (G_OBJECT (model->priv->query_model),
-							      G_CALLBACK (rhythmdb_property_model_row_inserted_cb),
-							      model);
-			g_signal_handlers_disconnect_by_func (G_OBJECT (model->priv->query_model),
-							      G_CALLBACK (rhythmdb_property_model_entry_removed_cb),
-							      model);
-			g_signal_handlers_disconnect_by_func (G_OBJECT (model->priv->query_model),
-							      G_CALLBACK (rhythmdb_property_model_prop_changed_cb),
-							      model);
-			g_object_unref (G_OBJECT (model->priv->query_model));
-		}
-		
-		model->priv->query_model = g_value_get_object (value);
-
-		if (model->priv->query_model) {
-			g_signal_connect_object (G_OBJECT (model->priv->query_model),
-						 "row_inserted",
-						 G_CALLBACK (rhythmdb_property_model_row_inserted_cb),
-						 model,
-						 0);
-			g_signal_connect_object (G_OBJECT (model->priv->query_model),
-						 "entry-removed",
-						 G_CALLBACK (rhythmdb_property_model_entry_removed_cb),
-						 model,
-						 0);
-			g_signal_connect_object (G_OBJECT (model->priv->query_model),
-						 "entry-prop-changed",
-						 G_CALLBACK (rhythmdb_property_model_prop_changed_cb),
-						 model,
-						 0);
-			g_object_ref (G_OBJECT (model->priv->query_model));
-		}
+		rhythmdb_property_model_set_query_model_internal (model, g_value_get_object (value));
 		break;
-	}
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
 		break;
 	}
 }
 
-static void 
+static void
 rhythmdb_property_model_get_property (GObject *object,
-				   guint prop_id,
-				   GValue *value,
-				   GParamSpec *pspec)
+				      guint prop_id,
+				      GValue *value,
+				      GParamSpec *pspec)
 {
 	RhythmDBPropertyModel *model = RHYTHMDB_PROPERTY_MODEL (object);
 
-	switch (prop_id)
-	{
+	switch (prop_id) {
 	case PROP_RHYTHMDB:
 		g_value_set_object (value, model->priv->db);
 		break;
-	case PROP_QUERY:
-		g_value_set_pointer (value, model->priv->query);
-		break;
 	case PROP_PROP:
 		g_value_set_int (value, model->priv->propid);
 		break;
@@ -374,20 +411,16 @@ rhythmdb_property_model_get_property (GObject *object,
 static void
 rhythmdb_property_model_init (RhythmDBPropertyModel *model)
 {
-	model->priv = g_new0 (RhythmDBPropertyModelPrivate, 1);
-
-	model->priv->property_memchunk = g_mem_chunk_new ("RhythmDBPropertyModel property memchunk",
-							  sizeof (RhythmDBPropertyModelEntry),
-							  1024, G_ALLOC_AND_FREE);
+	model->priv = RHYTHMDB_PROPERTY_MODEL_GET_PRIVATE (model);
 
 	model->priv->stamp = g_random_int ();
 
 	model->priv->properties = g_sequence_new (NULL);
 	model->priv->reverse_map = g_hash_table_new (g_str_hash, g_str_equal);
+	model->priv->entries = g_hash_table_new (g_direct_hash, g_direct_equal);
 
-	model->priv->all = g_mem_chunk_alloc (model->priv->property_memchunk);
-	model->priv->all->name = g_strdup (_("All"));
-	model->priv->all->sort_key = NULL;
+	model->priv->all = g_new0 (RhythmDBPropertyModelEntry, 1);
+	model->priv->all->string = rb_refstring_new (_("All"));
 }
 
 static void
@@ -406,58 +439,90 @@ rhythmdb_property_model_finalize (GObject *object)
 
 	g_return_if_fail (model->priv != NULL);
 
+	if (model->priv->syncing_id != 0)
+		g_source_remove (model->priv->syncing_id);
+
 	end_ptr = g_sequence_get_end_ptr (model->priv->properties);
 	for (ptr = g_sequence_get_begin_ptr (model->priv->properties); ptr != end_ptr;
 	     ptr = g_sequence_ptr_next (ptr)) {
 		RhythmDBPropertyModelEntry *prop = g_sequence_ptr_get_data (ptr);
-		g_free (prop->name);
-		g_free (prop->sort_key);
+		rb_refstring_unref (prop->string);
+		rb_refstring_unref (prop->sort_string);
 	}
 
-	g_mem_chunk_destroy (model->priv->property_memchunk);
 	g_hash_table_destroy (model->priv->reverse_map);
 	g_sequence_free (model->priv->properties);
 
-	g_free (model->priv);
+	g_hash_table_destroy (model->priv->entries);
+
+	if (model->priv->query_model != NULL) {
+		g_object_unref (model->priv->query_model);
+	}
 
 	G_OBJECT_CLASS (rhythmdb_property_model_parent_class)->finalize (object);
 }
 
 RhythmDBPropertyModel *
-rhythmdb_property_model_new (RhythmDB *db, RhythmDBPropType propid)
+rhythmdb_property_model_new (RhythmDB *db,
+			     RhythmDBPropType propid)
 {
 	return g_object_new (RHYTHMDB_TYPE_PROPERTY_MODEL, "db", db, "prop", propid, NULL);
 }
 
-static inline RhythmDBEntry *
-entry_from_tree_iter (GtkTreeModel *model, GtkTreeIter *iter)
-{
-	RhythmDBEntry *entry;
-	gtk_tree_model_get (model, iter, 0, &entry, -1);
-	return entry;
-}
-
 static void
 rhythmdb_property_model_row_inserted_cb (GtkTreeModel *model,
 					 GtkTreePath *path,
 					 GtkTreeIter *iter,
 					 RhythmDBPropertyModel *propmodel)
 {
-	RhythmDBEntry *entry = entry_from_tree_iter (model, iter);
-	rhythmdb_property_model_insert (propmodel, entry);
+	RhythmDBEntry *entry;
+	RhythmDBPropertyModelEntry *prop;
+
+	entry = rhythmdb_query_model_iter_to_entry (RHYTHMDB_QUERY_MODEL (model), iter);
+
+	prop = rhythmdb_property_model_insert (propmodel, entry);
+	rhythmdb_property_model_sync (propmodel);
+
+	rhythmdb_entry_unref (entry);
 }
 
 static void
-rhythmdb_property_model_prop_changed_cb (RhythmDB *db, RhythmDBEntry *entry,
-					 RhythmDBPropType prop, const GValue *old,
+rhythmdb_property_model_prop_changed_cb (RhythmDB *db,
+					 RhythmDBEntry *entry,
+					 RhythmDBPropType propid,
+					 const GValue *old,
 					 const GValue *new,
 					 RhythmDBPropertyModel *propmodel)
 {
-	if (prop != propmodel->priv->propid)
-		return;
+	if (propid == RHYTHMDB_PROP_HIDDEN) {
+		gboolean old_val = g_value_get_boolean (old);
+		gboolean new_val = g_value_get_boolean (new);
 
-	rhythmdb_property_model_delete_prop (propmodel, g_value_get_string (old));
-	rhythmdb_property_model_insert (propmodel, entry);
+		if (old_val != new_val) {
+			if (new_val == FALSE) {
+				g_assert (g_hash_table_remove (propmodel->priv->entries, entry));
+				rhythmdb_property_model_insert (propmodel, entry);
+			} else {
+				g_assert (g_hash_table_lookup (propmodel->priv->entries, entry) == NULL);
+
+				rhythmdb_property_model_delete (propmodel, entry);
+				g_hash_table_insert (propmodel->priv->entries, entry, GINT_TO_POINTER (1));
+			}
+		}
+	} else {
+		RhythmDBPropertyModelEntry *prop;
+
+		if (propid != propmodel->priv->propid)
+			return;
+
+		if (g_hash_table_lookup (propmodel->priv->entries, entry) != NULL)
+			return;
+
+		rhythmdb_property_model_delete_prop (propmodel, g_value_get_string (old));
+		prop = rhythmdb_property_model_insert (propmodel, entry);
+	}
+
+	rhythmdb_property_model_sync (propmodel);
 }
 
 static void
@@ -465,74 +530,82 @@ rhythmdb_property_model_entry_removed_cb (RhythmDBQueryModel *model,
 					  RhythmDBEntry *entry,
 					  RhythmDBPropertyModel *propmodel)
 {
+	if (g_hash_table_remove (propmodel->priv->entries, entry))
+		return;
+
 	rhythmdb_property_model_delete (propmodel, entry);
+	rhythmdb_property_model_sync (propmodel);
 }
 
 static gint
-rhythmdb_property_model_compare (RhythmDBPropertyModelEntry *a, RhythmDBPropertyModelEntry *b, RhythmDBPropertyModel *model)
+rhythmdb_property_model_compare (RhythmDBPropertyModelEntry *a,
+				 RhythmDBPropertyModelEntry *b,
+				 RhythmDBPropertyModel *model)
 {
-	return strcmp (a->sort_key, b->sort_key);
+	const char *a_str, *b_str;
+
+	a_str = rb_refstring_get_sort_key (a->sort_string);
+	b_str = rb_refstring_get_sort_key (b->sort_string);
+
+	return strcmp (a_str, b_str);
 }
 
-static void
-rhythmdb_property_model_insert (RhythmDBPropertyModel *model, RhythmDBEntry *entry)
-{
-	char *propstr;
-	char *sort_key;
-
-	propstr = g_strdup (rhythmdb_entry_get_string (entry, model->priv->propid));
-	sort_key = g_strdup (rhythmdb_entry_get_string (entry, model->priv->sort_propid));
-
-	rhythmdb_property_model_insert_prop (model, propstr, sort_key);
-	g_free (propstr);
-	g_free (sort_key);	
-}
-
-static void
-rhythmdb_property_model_insert_prop (RhythmDBPropertyModel *model,
-				     const char *propstr,
-				     const char *sort_key)
+static RhythmDBPropertyModelEntry *
+rhythmdb_property_model_insert (RhythmDBPropertyModel *model,
+				RhythmDBEntry *entry)
 {
 	RhythmDBPropertyModelEntry *prop;
 	GtkTreeIter iter;
 	GtkTreePath *path;
 	GSequencePtr ptr;
+	const char *propstr;
+
+	propstr = rhythmdb_entry_get_string (entry, model->priv->propid);
+
+	model->priv->all->refcount++;
 
 	if ((ptr = g_hash_table_lookup (model->priv->reverse_map, propstr))) {
 		prop = g_sequence_ptr_get_data (ptr);
 		prop->refcount++;
-		return;
+		rb_debug ("adding \"%s\": refcount %d", propstr, prop->refcount);
+		return prop;
 	}
+	rb_debug ("adding new property \"%s\"", propstr);
 
-	prop = g_mem_chunk_alloc (model->priv->property_memchunk);
-	prop->name = g_strdup (propstr);
-	prop->sort_key = g_strdup (sort_key);
+	prop = g_new0 (RhythmDBPropertyModelEntry, 1);
+	prop->string = rb_refstring_new (propstr);
+	prop->sort_string = rb_refstring_new (rhythmdb_entry_get_string (entry, model->priv->sort_propid));
 	prop->refcount = 1;
 
 	iter.stamp = model->priv->stamp;
 	ptr = g_sequence_insert_sorted (model->priv->properties, prop,
 					(GCompareDataFunc) rhythmdb_property_model_compare,
 					model);
-	g_hash_table_insert (model->priv->reverse_map, prop->name, ptr);
-	
+	g_hash_table_insert (model->priv->reverse_map,
+			     (gpointer)rb_refstring_get (prop->string),
+			     ptr);
+
 	iter.user_data = ptr;
 	path = rhythmdb_property_model_get_path (GTK_TREE_MODEL (model), &iter);
 	gtk_tree_model_row_inserted (GTK_TREE_MODEL (model), path, &iter);
 	gtk_tree_path_free (path);
+
+	return prop;
 }
 
-void
+static void
 rhythmdb_property_model_delete (RhythmDBPropertyModel *model,
 				RhythmDBEntry *entry)
 {
-	char *propstr;
+	const char *propstr;
 
-	propstr = g_strdup (rhythmdb_entry_get_string (entry, model->priv->propid));
-	
+	if (g_hash_table_lookup (model->priv->entries, entry))
+		return;
+
+	propstr = rhythmdb_entry_get_string (entry, model->priv->propid);
 	rhythmdb_property_model_delete_prop (model, propstr);
-	g_free (propstr);
 }
-	
+
 static void
 rhythmdb_property_model_delete_prop (RhythmDBPropertyModel *model,
 				     const char *propstr)
@@ -544,6 +617,8 @@ rhythmdb_property_model_delete_prop (RhythmDBPropertyModel *model,
 
 	g_assert ((ptr = g_hash_table_lookup (model->priv->reverse_map, propstr)));
 
+	model->priv->all->refcount--;
+
 	prop = g_sequence_ptr_get_data (ptr);
 	rb_debug ("deleting \"%s\": refcount: %d", propstr, prop->refcount);
 	prop->refcount--;
@@ -558,26 +633,36 @@ rhythmdb_property_model_delete_prop (RhythmDBPropertyModel *model,
 	gtk_tree_model_row_deleted (GTK_TREE_MODEL (model), path);
 	gtk_tree_path_free (path);
 	g_sequence_remove (ptr);
-	g_hash_table_remove (model->priv->reverse_map, prop->name);
-	g_free (prop->name);
-	g_free (prop->sort_key);
-	g_mem_chunk_free (model->priv->property_memchunk, prop);
+	g_hash_table_remove (model->priv->reverse_map, propstr);
+	rb_refstring_unref (prop->string);
+	rb_refstring_unref (prop->sort_string);
+	g_free (prop);
 	return;
 }
 
 gboolean
 rhythmdb_property_model_iter_from_string (RhythmDBPropertyModel *model,
-					  const char *name, GtkTreeIter *iter)
+					  const char *name,
+					  GtkTreeIter *iter)
 {
 	GSequencePtr ptr;
 
-	ptr = g_hash_table_lookup (model->priv->reverse_map, name);
+	if (name == NULL) {
+		if (iter) {
+			iter->stamp = model->priv->stamp;
+			iter->user_data = model->priv->all;
+		}
+		return TRUE;
+	}
 
+	ptr = g_hash_table_lookup (model->priv->reverse_map, name);
 	if (!ptr)
 		return FALSE;
 
-	iter->stamp = model->priv->stamp;
-	iter->user_data = ptr;
+	if (iter) {
+		iter->stamp = model->priv->stamp;
+		iter->user_data = ptr;
+	}
 
 	return TRUE;
 }
@@ -595,14 +680,16 @@ rhythmdb_property_model_get_n_columns (GtkTreeModel *tree_model)
 }
 
 static GType
-rhythmdb_property_model_get_column_type (GtkTreeModel *tree_model, int index)
+rhythmdb_property_model_get_column_type (GtkTreeModel *tree_model,
+					 int index)
 {
-	switch (index)
-	{
-	case 0:
+	switch (index) {
+	case RHYTHMDB_PROPERTY_MODEL_COLUMN_TITLE:
 		return G_TYPE_STRING;
-	case 1:
+	case RHYTHMDB_PROPERTY_MODEL_COLUMN_PRIORITY:
 		return G_TYPE_BOOLEAN;
+	case RHYTHMDB_PROPERTY_MODEL_COLUMN_NUMBER:
+		return G_TYPE_UINT;
 	default:
 		g_assert_not_reached ();
 		return G_TYPE_INVALID;
@@ -610,8 +697,9 @@ rhythmdb_property_model_get_column_type (GtkTreeModel *tree_model, int index)
 }
 
 static gboolean
-rhythmdb_property_model_get_iter (GtkTreeModel *tree_model, GtkTreeIter *iter,
-			       GtkTreePath *path)
+rhythmdb_property_model_get_iter (GtkTreeModel *tree_model,
+				  GtkTreeIter *iter,
+				  GtkTreePath *path)
 {
 	RhythmDBPropertyModel *model = RHYTHMDB_PROPERTY_MODEL (tree_model);
 	guint index;
@@ -639,13 +727,17 @@ rhythmdb_property_model_get_iter (GtkTreeModel *tree_model, GtkTreeIter *iter,
 
 static GtkTreePath *
 rhythmdb_property_model_get_path (GtkTreeModel *tree_model,
-			       GtkTreeIter  *iter)
+				  GtkTreeIter  *iter)
 {
 	RhythmDBPropertyModel *model = RHYTHMDB_PROPERTY_MODEL (tree_model);
 	GtkTreePath *path;
 
 	g_return_val_if_fail (iter->stamp == model->priv->stamp, NULL);
 
+	if (iter->user_data == model->priv->all) {
+		return gtk_tree_path_new_first ();
+	}
+
 	if (g_sequence_ptr_is_end (iter->user_data))
 		return NULL;
 
@@ -653,47 +745,56 @@ rhythmdb_property_model_get_path (GtkTreeModel *tree_model,
 	if (iter->user_data == model->priv->all)
 		gtk_tree_path_append_index (path, 0);
 	else
-		gtk_tree_path_append_index (path, g_sequence_ptr_get_position (iter->user_data)+1);
+		gtk_tree_path_append_index (path, g_sequence_ptr_get_position (iter->user_data) + 1);
 	return path;
 }
 
 static void
-rhythmdb_property_model_get_value (GtkTreeModel *tree_model, GtkTreeIter *iter,
-				   gint column, GValue *value)
+rhythmdb_property_model_get_value (GtkTreeModel *tree_model,
+				   GtkTreeIter *iter,
+				   gint column,
+				   GValue *value)
 {
 	RhythmDBPropertyModel *model = RHYTHMDB_PROPERTY_MODEL (tree_model);
-	RhythmDBPropertyModelEntry *prop;
 
-	g_return_if_fail (!g_sequence_ptr_is_end (iter->user_data));
 	g_return_if_fail (model->priv->stamp == iter->stamp);
 
 	if (iter->user_data == model->priv->all) {
-		switch (column)
-		{
-		case 0:
+		switch (column) {
+		case RHYTHMDB_PROPERTY_MODEL_COLUMN_TITLE:
 			g_value_init (value, G_TYPE_STRING);
-			g_value_set_string (value, model->priv->all->name);
+			g_value_set_string (value, rb_refstring_get (model->priv->all->string));
 			break;
-		case 1:
+		case RHYTHMDB_PROPERTY_MODEL_COLUMN_PRIORITY:
 			g_value_init (value, G_TYPE_BOOLEAN);
 			g_value_set_boolean (value, TRUE);
 			break;
+		case RHYTHMDB_PROPERTY_MODEL_COLUMN_NUMBER:
+			g_value_init (value, G_TYPE_UINT);
+			g_value_set_uint (value, model->priv->all->refcount);
+			break;
 		default:
 			g_assert_not_reached ();
 		}
 	} else {
+		RhythmDBPropertyModelEntry *prop;
+
+		g_return_if_fail (!g_sequence_ptr_is_end (iter->user_data));
+
 		prop = g_sequence_ptr_get_data (iter->user_data);
-		
-		switch (column)
-		{
-		case 0:
+		switch (column) {
+		case RHYTHMDB_PROPERTY_MODEL_COLUMN_TITLE:
 			g_value_init (value, G_TYPE_STRING);
-			g_value_set_string (value, prop->name);
+			g_value_set_string (value, rb_refstring_get (prop->string));
 			break;
-		case 1:
+		case RHYTHMDB_PROPERTY_MODEL_COLUMN_PRIORITY:
 			g_value_init (value, G_TYPE_BOOLEAN);
 			g_value_set_boolean (value, prop == model->priv->all);
 			break;
+		case RHYTHMDB_PROPERTY_MODEL_COLUMN_NUMBER:
+			g_value_init (value, G_TYPE_UINT);
+			g_value_set_uint (value, prop->refcount);
+			break;
 		default:
 			g_assert_not_reached ();
 		}
@@ -702,16 +803,18 @@ rhythmdb_property_model_get_value (GtkTreeModel *tree_model, GtkTreeIter *iter,
 
 static gboolean
 rhythmdb_property_model_iter_next (GtkTreeModel  *tree_model,
-				GtkTreeIter   *iter)
+				   GtkTreeIter   *iter)
 {
 	RhythmDBPropertyModel *model = RHYTHMDB_PROPERTY_MODEL (tree_model);
 
 	g_return_val_if_fail (iter->stamp == model->priv->stamp, FALSE);
 
-	if (iter->user_data == model->priv->all)
+	if (iter->user_data == model->priv->all) {
 		iter->user_data = g_sequence_get_begin_ptr (model->priv->properties);
-	else
+	} else {
+		g_return_val_if_fail (!g_sequence_ptr_is_end (iter->user_data), FALSE);
 		iter->user_data = g_sequence_ptr_next (iter->user_data);
+	}
 
 	return !g_sequence_ptr_is_end (iter->user_data);
 }
@@ -756,8 +859,9 @@ rhythmdb_property_model_iter_n_children (GtkTreeModel *tree_model,
 
 static gboolean
 rhythmdb_property_model_iter_nth_child (GtkTreeModel *tree_model,
-				     GtkTreeIter *iter, GtkTreeIter *parent,
-				     gint n)
+					GtkTreeIter *iter,
+					GtkTreeIter *parent,
+					gint n)
 {
 	RhythmDBPropertyModel *model = RHYTHMDB_PROPERTY_MODEL (tree_model);
 	GSequencePtr child;
@@ -767,7 +871,7 @@ rhythmdb_property_model_iter_nth_child (GtkTreeModel *tree_model,
 
 	if (n != 0) {
 		child = g_sequence_get_ptr_at_pos (model->priv->properties, n);
-		
+
 		if (g_sequence_ptr_is_end (child))
 			return FALSE;
 		iter->user_data = child;
@@ -796,7 +900,7 @@ rhythmdb_property_model_row_draggable (RbTreeDragSource *dragsource,
 }
 
 static gboolean
-rhythmdb_property_model_drag_data_delete (RbTreeDragSource *dragsource, 
+rhythmdb_property_model_drag_data_delete (RbTreeDragSource *dragsource,
 					  GList *paths)
 {
 	/* not supported */
@@ -804,25 +908,26 @@ rhythmdb_property_model_drag_data_delete (RbTreeDragSource *dragsource,
 }
 
 /*Going through hoops to avoid nested functions*/
-struct QueryModelCbStruct { 
-	RhythmDB* db; 
-	GString* reply; 
-}; 
+struct QueryModelCbStruct {
+	RhythmDB *db;
+	GString *reply;
+};
 
-static gboolean 
+static gboolean
 query_model_cb (GtkTreeModel *query_model,
- 		GtkTreePath *path, GtkTreeIter *iter,
- 		struct QueryModelCbStruct *data) 
+ 		GtkTreePath *path,
+		GtkTreeIter *iter,
+ 		struct QueryModelCbStruct *data)
 {
- 	const char* uri;
- 	RhythmDBEntry* entry;
-	
+ 	const char *uri;
+ 	RhythmDBEntry *entry;
+
  	gtk_tree_model_get (query_model, iter, 0, &entry, -1);
  	uri = rhythmdb_entry_get_string (entry, RHYTHMDB_PROP_LOCATION);
  	g_string_append (data->reply, uri);
  	g_string_append (data->reply, "\r\n");
-	
- 	rhythmdb_entry_unref (data->db, entry);
+
+ 	rhythmdb_entry_unref (entry);
  	return FALSE;
 }
 
@@ -845,91 +950,105 @@ rhythmdb_property_model_drag_data_get (RbTreeDragSource *dragsource,
 	case RHYTHMDB_PROP_ARTIST:
 		drag_target_list = rhythmdb_property_model_artist_drag_target_list;
 		break;
+	case RHYTHMDB_PROP_LOCATION:
+		drag_target_list = rhythmdb_property_model_location_drag_target_list;
+		break;
 	default:
 		g_assert_not_reached ();
 	}
 
 	if (!gtk_target_list_find (drag_target_list,
-				   selection_data->target, 
+				   selection_data->target,
 				   &target)) {
 		return FALSE;
 	}
 
 	if (target == TARGET_URIS) {
 		RhythmDB *db = model->priv->db;
- 		GtkTreeModel* query_model = GTK_TREE_MODEL (rhythmdb_query_model_new_empty(db));
+ 		RhythmDBQueryModel *query_model;
  		GString* reply = g_string_new ("");
  		GtkTreeIter iter;
- 		gboolean is_all;
+ 		gboolean is_all = FALSE;
  		struct QueryModelCbStruct tmp;
 		GtkTreePath *path;
+		GCompareDataFunc sort_func = NULL;
+		gpointer sort_data;
+		gboolean sort_reverse;
+
+		query_model = rhythmdb_query_model_new_empty (db);
+		/* FIXME the sort order on the query model at this point is usually
+		 * not the user's selected sort order.
+		 */
+		g_object_get (G_OBJECT (model->priv->query_model),
+			      "sort-func", &sort_func,
+			      "sort-data", &sort_data,
+			      "sort-reverse", &sort_reverse,
+			      NULL);
+		rhythmdb_query_model_set_sort_order (RHYTHMDB_QUERY_MODEL (query_model),
+						     sort_func, GUINT_TO_POINTER (sort_data), NULL, sort_reverse);
 
 		rb_debug ("getting drag data as uri list");
 		/* check if first selected row is 'All' */
 		path = gtk_tree_row_reference_get_path (paths->data);
-		gtk_tree_model_get_iter (GTK_TREE_MODEL (model), &iter, path);
-		gtk_tree_model_get (GTK_TREE_MODEL (model), &iter,
-				    RHYTHMDB_PROPERTY_MODEL_COLUMN_PRIORITY,
-				    &is_all, -1);
+		if (path && gtk_tree_model_get_iter (GTK_TREE_MODEL (model), &iter, path))
+			gtk_tree_model_get (GTK_TREE_MODEL (model), &iter,
+					    RHYTHMDB_PROPERTY_MODEL_COLUMN_PRIORITY,
+					    &is_all, -1);
 		gtk_tree_path_free (path);
 		if (is_all) {
- 			/*filter out radios*/
- 			rhythmdb_do_full_query (db, query_model,
- 						RHYTHMDB_QUERY_PROP_EQUALS,
- 						RHYTHMDB_PROP_TYPE, 
- 						RHYTHMDB_ENTRY_TYPE_SONG,
- 						RHYTHMDB_QUERY_END);
+			g_object_set (query_model,
+				      "base-model", model->priv->query_model,
+				      NULL);
 		} else {
  			GList *row;
-			GPtrArray* subquery = g_ptr_array_new();
- 
+			GPtrArray *subquery = g_ptr_array_new ();
+
  			for (row = paths; row; row = row->next) {
  				char* name;
 				path = gtk_tree_row_reference_get_path (row->data);
- 				gtk_tree_model_get_iter (GTK_TREE_MODEL (model), &iter, path);
- 				gtk_tree_model_get (GTK_TREE_MODEL (model), &iter,
- 						    RHYTHMDB_PROPERTY_MODEL_COLUMN_TITLE, 
-						    &name, -1);
- 				if (row->next) {
- 					rhythmdb_query_append (db, subquery,
- 							       RHYTHMDB_QUERY_PROP_EQUALS,
- 							       model->priv->propid, name,
- 							       RHYTHMDB_QUERY_DISJUNCTION,
- 							       RHYTHMDB_QUERY_END);
- 				} else {
- 					rhythmdb_query_append (db, subquery,
- 							       RHYTHMDB_QUERY_PROP_EQUALS,
- 							       model->priv->propid, name,
-							       RHYTHMDB_QUERY_END);
- 				}
- 
+ 				if (path && gtk_tree_model_get_iter (GTK_TREE_MODEL (model), &iter, path)) {
+	 				gtk_tree_model_get (GTK_TREE_MODEL (model), &iter,
+	 						    RHYTHMDB_PROPERTY_MODEL_COLUMN_TITLE,
+							    &name, -1);
+	 				if (row == paths) {
+	 					rhythmdb_query_append (db, subquery,
+	 							       RHYTHMDB_QUERY_PROP_EQUALS,
+	 							       model->priv->propid, name,
+								       RHYTHMDB_QUERY_END);
+					} else {
+	 					rhythmdb_query_append (db, subquery,
+	 							       RHYTHMDB_QUERY_DISJUNCTION,
+	 							       RHYTHMDB_QUERY_PROP_EQUALS,
+	 							       model->priv->propid, name,
+	 							       RHYTHMDB_QUERY_END);
+	 				}
+				}
+
 				gtk_tree_path_free (path);
  				g_free (name);
  			}
- 			/*filter out radios*/
- 			rhythmdb_do_full_query (db, query_model,
- 						RHYTHMDB_QUERY_PROP_EQUALS,
- 						RHYTHMDB_PROP_TYPE, 
- 						RHYTHMDB_ENTRY_TYPE_SONG,
- 						RHYTHMDB_QUERY_SUBQUERY,
- 						subquery,
- 						RHYTHMDB_QUERY_END);
+
+			g_object_set (query_model,
+				      "query", subquery,
+				      "base-model", model->priv->query_model,
+				      NULL);
+			rhythmdb_query_free (subquery);
 		}
- 		
-		tmp.db = db; 
+
+		tmp.db = db;
  		tmp.reply = reply;
  		/* Too bad that we're on the main thread. Why doesn't gtk call us async?
- 		 * How does file-roller manage? - it seems it refuses the drop when it isn't 
-		 * done unpacking. In which case, we should tweak the drop acknowledgement, 
-		 * and prepare the query using do_full_query_async. The query would be 
+ 		 * How does file-roller manage? - it seems it refuses the drop when it isn't
+		 * done unpacking. In which case, we should tweak the drop acknowledgement,
+		 * and prepare the query using do_full_query_async. The query would be
 		 * hooked to the drag context.
 		 */
- 		gtk_tree_model_foreach (query_model,
- 					(GtkTreeModelForeachFunc)query_model_cb,
+ 		gtk_tree_model_foreach (GTK_TREE_MODEL (query_model),
+ 					(GtkTreeModelForeachFunc) query_model_cb,
  					&tmp);
- 		
- 		g_object_unref (query_model);
- 
+
+		g_object_unref (query_model);
+
  		gtk_selection_data_set (selection_data,
  		                        selection_data->target,
  		                        8, (guchar *)reply->str,
@@ -940,7 +1059,7 @@ rhythmdb_property_model_drag_data_get (RbTreeDragSource *dragsource,
 		char* title;
 		GList *p;
 		GString* reply = g_string_new ("");
- 
+
 		rb_debug ("getting drag data as list of property values");
 
 		for (p = paths; p; p = p->next) {
@@ -948,13 +1067,14 @@ rhythmdb_property_model_drag_data_get (RbTreeDragSource *dragsource,
 			GtkTreePath *path;
 
 			path = gtk_tree_row_reference_get_path (p->data);
-			gtk_tree_model_get_iter (GTK_TREE_MODEL (model), &iter, path);
-			gtk_tree_model_get (GTK_TREE_MODEL (model), &iter,
-					    RHYTHMDB_PROPERTY_MODEL_COLUMN_TITLE, &title, -1);
-			g_string_append (reply, title);
-			if (p->next)
-				g_string_append (reply, "\r\n");
-			g_free (title);
+			if (path && gtk_tree_model_get_iter (GTK_TREE_MODEL (model), &iter, path)) {
+				gtk_tree_model_get (GTK_TREE_MODEL (model), &iter,
+						    RHYTHMDB_PROPERTY_MODEL_COLUMN_TITLE, &title, -1);
+				g_string_append (reply, title);
+				if (p->next)
+					g_string_append (reply, "\r\n");
+				g_free (title);
+			}
 			gtk_tree_path_free (path);
 		}
 		gtk_selection_data_set (selection_data,
@@ -967,8 +1087,9 @@ rhythmdb_property_model_drag_data_get (RbTreeDragSource *dragsource,
 	return TRUE;
 }
 
-void			
-rhythmdb_property_model_enable_drag (RhythmDBPropertyModel *model, GtkTreeView *view)
+void
+rhythmdb_property_model_enable_drag (RhythmDBPropertyModel *model,
+				     GtkTreeView *view)
 {
 	const GtkTargetEntry *targets;
 	gint n_elements;
@@ -986,12 +1107,66 @@ rhythmdb_property_model_enable_drag (RhythmDBPropertyModel *model, GtkTreeView *
 		targets = targets_artist;
 		n_elements = G_N_ELEMENTS (targets_artist);
 		break;
+	case RHYTHMDB_PROP_LOCATION:
+		targets = targets_location;
+		n_elements = G_N_ELEMENTS (targets_location);
+		break;
 	default:
 		g_assert_not_reached ();
 	}
-	
+
 	rb_tree_dnd_add_drag_source_support (view,
 					     GDK_BUTTON1_MASK,
 					     targets, n_elements,
 					     GDK_ACTION_COPY);
 }
+
+static gboolean
+rhythmdb_property_model_perform_sync (RhythmDBPropertyModel *model)
+{
+	GtkTreeIter iter;
+	GtkTreePath *path;
+
+	GDK_THREADS_ENTER ();
+
+	iter.stamp = model->priv->stamp;
+	iter.user_data = model->priv->all;
+	path = rhythmdb_property_model_get_path (GTK_TREE_MODEL (model), &iter);
+	gtk_tree_model_row_changed (GTK_TREE_MODEL (model), path, &iter);
+	gtk_tree_path_free (path);
+
+	model->priv->syncing_id = 0;
+	GDK_THREADS_LEAVE ();
+	return FALSE;
+}
+
+static void
+rhythmdb_property_model_sync (RhythmDBPropertyModel *model)
+{
+	if (model->priv->syncing_id != 0)
+		return;
+
+	model->priv->syncing_id = g_idle_add ((GSourceFunc)rhythmdb_property_model_perform_sync, model);
+}
+
+/* This should really be standard. */
+#define ENUM_ENTRY(NAME, DESC) { NAME, "" #NAME "", DESC }
+
+GType
+rhythmdb_property_model_column_get_type (void)
+{
+	static GType etype = 0;
+
+	if (etype == 0)	{
+		static const GEnumValue values[] = {
+			ENUM_ENTRY (RHYTHMDB_PROPERTY_MODEL_COLUMN_TITLE, "Property title"),
+			ENUM_ENTRY (RHYTHMDB_PROPERTY_MODEL_COLUMN_PRIORITY, "Value priority"),
+			ENUM_ENTRY (RHYTHMDB_PROPERTY_MODEL_COLUMN_NUMBER, "Track count"),
+			{ 0, 0, 0 }
+		};
+
+		etype = g_enum_register_static ("RhythmDBPropertyModelColumn", values);
+	}
+
+	return etype;
+}
