@@ -1,5 +1,5 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*-
- * 
+ *
  *  arch-tag: Implementation of search entry/browse toggle container
  *
  *  Copyright (C) 2003 Jorn Baayen <jorn@nl.linux.org>
@@ -17,7 +17,7 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA.
  *
  */
 
@@ -29,7 +29,6 @@
 #include <glib/gi18n.h>
 #include <gtk/gtk.h>
 
-#include "disclosure-widget.h"
 #include "rb-source-header.h"
 #include "rb-stock-icons.h"
 #include "rb-preferences.h"
@@ -55,15 +54,24 @@ static void rb_source_header_filter_changed_cb (RBSource *source,
 static void rb_source_header_search_cb (RBSearchEntry *search,
 					const char *text,
 					RBSourceHeader *header);
-static void rb_source_header_disclosure_toggled_cb (GObject *object,
-						    GParamSpec *param_spec,
-						    gpointer data);
 static void rb_source_header_search_activate_cb (RBSearchEntry *search,
 						 RBSourceHeader *header);
 static void rb_source_header_view_browser_changed_cb (GtkAction *action,
 						      RBSourceHeader *header);
 static void rb_source_header_source_weak_destroy_cb (RBSourceHeader *header, RBSource *source);
 
+typedef struct {
+	gboolean disclosed;
+	char     *search_text;
+}  SourceState;
+
+static void
+sourcestate_free (SourceState *state)
+{
+        g_free (state->search_text);
+        g_free (state);
+}
+
 struct RBSourceHeaderPrivate
 {
 	RBSource *selected_source;
@@ -76,7 +84,6 @@ struct RBSourceHeaderPrivate
 
 	GtkWidget *search;
 	GtkWidget *search_bar;
-	GtkWidget *disclosure;
 
 	guint browser_notify_id;
 	guint search_notify_id;
@@ -85,7 +92,8 @@ struct RBSourceHeaderPrivate
 	gboolean disclosed;
 	char *browser_key;
 
-	GHashTable *source_search_text;
+	GHashTable *source_states;
+
 };
 
 #define RB_SOURCE_HEADER_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), RB_TYPE_SOURCE_HEADER, RBSourceHeaderPrivate))
@@ -145,7 +153,7 @@ ui_manager_add_widget_cb (GtkUIManager *ui_manager,
 			gtk_widget_show (header->priv->search_bar);
 			gtk_table_attach (GTK_TABLE (header),
 					  header->priv->search_bar,
-					  1, 3, 0, 1, 
+					  1, 3, 0, 1,
 					  GTK_EXPAND | GTK_FILL,
 					  GTK_EXPAND | GTK_FILL,
 					  5, 0);
@@ -227,8 +235,8 @@ rb_source_header_init (RBSourceHeader *header)
 
 	ebox = GTK_EVENT_BOX (gtk_event_box_new ());
 	header->priv->search = GTK_WIDGET (rb_search_entry_new ());
-	gtk_tooltips_set_tip (GTK_TOOLTIPS (header->priv->tooltips), 
-			      GTK_WIDGET (ebox), 
+	gtk_tooltips_set_tip (GTK_TOOLTIPS (header->priv->tooltips),
+			      GTK_WIDGET (ebox),
 			      _("Filter music display by genre, artist, album, or title"),
 			      NULL);
 	gtk_container_add (GTK_CONTAINER (ebox), GTK_WIDGET (header->priv->search));
@@ -238,28 +246,17 @@ rb_source_header_init (RBSourceHeader *header)
 	g_signal_connect_object (G_OBJECT (header->priv->search), "activate",
 				 G_CALLBACK (rb_source_header_search_activate_cb), header, 0);
 
-	header->priv->disclosure = cddb_disclosure_new (_("Show _Browser"),
-							_("Hide _Browser"));
-	gtk_widget_set_sensitive (header->priv->disclosure, FALSE);
-	g_signal_connect_object (G_OBJECT (header->priv->disclosure), "notify::expanded",
-				 G_CALLBACK (rb_source_header_disclosure_toggled_cb), header, 0);
-
-#if 0	
-	gtk_table_attach_defaults (GTK_TABLE (header),
-			           header->priv->disclosure, 2, 3, 0, 1);
-#endif
-
 	align = gtk_alignment_new (1.0, 0.5, 1.0, 1.0);
 	gtk_container_add (GTK_CONTAINER (align), GTK_WIDGET (ebox));
 	gtk_table_attach (GTK_TABLE (header),
 			  align,
-			  0, 1, 0, 1, 
+			  0, 1, 0, 1,
 			  GTK_EXPAND | GTK_FILL,
 			  GTK_EXPAND | GTK_FILL,
 			  5, 0);
 
-	header->priv->source_search_text = g_hash_table_new_full (g_direct_hash, g_direct_equal,
-								  NULL, (GDestroyNotify)g_free);
+	header->priv->source_states = g_hash_table_new_full (g_direct_hash, g_direct_equal,
+							    NULL, (GDestroyNotify)sourcestate_free);
 }
 
 static void
@@ -282,10 +279,10 @@ rb_source_header_finalize (GObject *object)
 
 	g_return_if_fail (header->priv != NULL);
 
-	g_hash_table_foreach (header->priv->source_search_text,
+	g_hash_table_foreach (header->priv->source_states,
 			      (GHFunc) rb_source_header_source_weak_unref,
 			      header);
-	g_hash_table_destroy (header->priv->source_search_text);
+	g_hash_table_destroy (header->priv->source_states);
 
 	g_free (header->priv->browser_key);
 
@@ -322,18 +319,30 @@ rb_source_header_set_source_internal (RBSourceHeader *header,
 	rb_debug ("selected source %p", source);
 
 	if (header->priv->selected_source != NULL) {
-		const char *text = g_hash_table_lookup (header->priv->source_search_text,
-							header->priv->selected_source);
-			
+		SourceState *source_state;
+		char        *text;
+		gboolean    disclosed;
+
+		source_state = g_hash_table_lookup (header->priv->source_states,
+						    header->priv->selected_source);
+
+		if (source_state) {
+			text = g_strdup (source_state->search_text);
+			disclosed = source_state->disclosed;
+		} else {
+			text = NULL;
+			disclosed = FALSE;
+		}
+
 		g_free (header->priv->browser_key);
 		header->priv->browser_key = rb_source_get_browser_key (header->priv->selected_source);
-	
+
 		rb_search_entry_set_text (RB_SEARCH_ENTRY (header->priv->search), text);
 		g_signal_connect_object (G_OBJECT (header->priv->selected_source),
 					 "filter_changed",
 					 G_CALLBACK (rb_source_header_filter_changed_cb),
 					 header, 0);
-	
+
 		gtk_widget_set_sensitive (GTK_WIDGET (header->priv->search),
 					  rb_source_can_search (header->priv->selected_source));
 		header->priv->have_search = rb_source_can_search (header->priv->selected_source);
@@ -343,9 +352,9 @@ rb_source_header_set_source_internal (RBSourceHeader *header,
 		else if (header->priv->browser_key)
 			header->priv->disclosed = eel_gconf_get_boolean (header->priv->browser_key);
 		else
-			/* FIXME: remember the previous state of the source*/
-			header->priv->disclosed = FALSE;
-	
+			/* restore the previous state of the source*/
+			header->priv->disclosed = disclosed;
+
 		if (!header->priv->have_browser && !header->priv->have_search)
 			gtk_widget_hide (GTK_WIDGET (header));
 		else
@@ -389,7 +398,7 @@ rb_source_header_set_property (GObject *object,
 	}
 }
 
-static void 
+static void
 rb_source_header_get_property (GObject *object,
 			      guint prop_id,
 			      GValue *value,
@@ -455,7 +464,41 @@ rb_source_header_filter_changed_cb (RBSource *source,
 static void
 rb_source_header_source_weak_destroy_cb (RBSourceHeader *header, RBSource *source)
 {
-	g_hash_table_remove (header->priv->source_search_text, source);
+	g_hash_table_remove (header->priv->source_states, source);
+}
+
+static void
+rb_source_state_sync (RBSourceHeader *header,
+		      gboolean set_text,
+		      const char *text,
+		      gboolean set_disclosure,
+		      gboolean disclosed)
+{
+	SourceState *old_state;
+
+	old_state = g_hash_table_lookup (header->priv->source_states,
+					 header->priv->selected_source);
+
+	if (old_state) {
+		if (set_text)
+			old_state->search_text = text ? g_strdup (text) : NULL;
+		if (set_disclosure)
+			old_state->disclosed = disclosed;
+	} else {
+		SourceState *new_state;
+
+		/* if we haven't seen the source before, monitor it for deletion */
+		g_object_weak_ref (G_OBJECT (header->priv->selected_source),
+				   (GWeakNotify)rb_source_header_source_weak_destroy_cb,
+				   header);
+
+		new_state = g_new (SourceState, 1);
+		new_state->search_text = text ? g_strdup (text) : NULL;
+		new_state->disclosed = disclosed;
+		g_hash_table_insert (header->priv->source_states,
+				     header->priv->selected_source,
+				     new_state);
+	}
 }
 
 static void
@@ -463,18 +506,11 @@ rb_source_header_search_cb (RBSearchEntry *search,
 			    const char *text,
 			    RBSourceHeader *header)
 {
+
 	rb_debug  ("searching for \"%s\"", text);
 
-	/* if we haven't seen the source before, monitor it for deletion */
-	if (g_hash_table_lookup (header->priv->source_search_text, header->priv->selected_source) == NULL) {
-		g_object_weak_ref (G_OBJECT (header->priv->selected_source),
-				   (GWeakNotify)rb_source_header_source_weak_destroy_cb,
-				   header);
-	}
+	rb_source_state_sync (header, TRUE, text, FALSE, FALSE);
 
-	g_hash_table_insert (header->priv->source_search_text,
-			     header->priv->selected_source,
-			     g_strdup (text));
 	rb_source_search (header->priv->selected_source, text);
 	rb_source_header_sync_control_state (header);
 }
@@ -486,40 +522,33 @@ rb_source_header_clear_search (RBSourceHeader *header)
 
 	if (!rb_search_entry_searching (RB_SEARCH_ENTRY (header->priv->search)))
 	    return;
-	
+
 	if (header->priv->selected_source) {
 		rb_source_search (header->priv->selected_source, NULL);
-		g_hash_table_remove (header->priv->source_search_text, 
-				     header->priv->selected_source);
+		rb_source_state_sync (header, TRUE, NULL, FALSE, FALSE);
+
 	}
 	rb_search_entry_clear (RB_SEARCH_ENTRY (header->priv->search));
 	rb_source_header_sync_control_state (header);
 }
 
-static void
-rb_source_header_disclosure_toggled_cb (GObject *object,
-					GParamSpec *param_spec,
-					gpointer data)
-{
-	RBSourceHeader *header = RB_SOURCE_HEADER (data);
-	GtkExpander *expander = GTK_EXPANDER (object);
-
-	header->priv->disclosed = gtk_expander_get_expanded (expander);
-
-	if (header->priv->browser_key)
-		eel_gconf_set_boolean (header->priv->browser_key, 
-				       header->priv->disclosed);
-
-	rb_source_header_sync_control_state (header);
-}
-
 static void
 rb_source_header_view_browser_changed_cb (GtkAction *action,
 					  RBSourceHeader *header)
 {
 	rb_debug ("got view browser toggle");
-	gtk_expander_set_expanded (GTK_EXPANDER (header->priv->disclosure),
-				   gtk_toggle_action_get_active (GTK_TOGGLE_ACTION (action)));
+	header->priv->disclosed = gtk_toggle_action_get_active (GTK_TOGGLE_ACTION (action));
+
+	if (header->priv->browser_key)
+		eel_gconf_set_boolean (header->priv->browser_key,
+				       header->priv->disclosed);
+	else {
+		rb_source_state_sync (header, FALSE, NULL, TRUE, header->priv->disclosed);
+	}
+
+	rb_debug ("got view browser toggle");
+
+	rb_source_header_sync_control_state (header);
 }
 
 void
@@ -530,8 +559,6 @@ rb_source_header_sync_control_state (RBSourceHeader *header)
 	GtkAction *viewall_action;
 	gboolean not_small = !eel_gconf_get_boolean (CONF_UI_SMALL_DISPLAY);
 
-	gtk_widget_set_sensitive (header->priv->disclosure,
-				  header->priv->have_browser);
 	viewbrowser_action = gtk_action_group_get_action (header->priv->actiongroup,
 							  "ViewBrowser");
 	g_object_set (G_OBJECT (viewbrowser_action), "sensitive",
@@ -545,8 +572,6 @@ rb_source_header_sync_control_state (RBSourceHeader *header)
 	g_object_set (G_OBJECT (viewall_action), "sensitive",
 		      (header->priv->have_browser || header->priv->have_search) && not_small, NULL);
 
-	gtk_expander_set_expanded (GTK_EXPANDER (header->priv->disclosure),
-				   header->priv->disclosed);
 	gtk_toggle_action_set_active (GTK_TOGGLE_ACTION (viewbrowser_action),
 				      header->priv->disclosed);
 
