@@ -19,12 +19,13 @@
  *
  */
 
-#include <config.h>
-#include <gtk/gtk.h>
-#include <libgnome/gnome-i18n.h>
-#include <libgnomevfs/gnome-vfs-uri.h>
-#include <libxml/tree.h>
+#include "config.h"
+
 #include <string.h>
+#include <libxml/tree.h>
+#include <glib/gi18n.h>
+#include <gtk/gtk.h>
+#include <libgnomevfs/gnome-vfs-uri.h>
 
 #include "rb-auto-playlist-source.h"
 #include "rb-library-browser.h"
@@ -34,7 +35,6 @@
 #include "rb-stock-icons.h"
 #include "rb-playlist-xml.h"
 
-
 static GObject *rb_auto_playlist_source_constructor (GType type, guint n_construct_properties,
 						      GObjectConstructParam *construct_properties);
 static void rb_auto_playlist_source_finalize (GObject *object);
@@ -51,8 +51,8 @@ static GList *impl_get_search_actions (RBSource *source);
 static void impl_save_contents_to_xml (RBPlaylistSource *source,
 				       xmlNodePtr node);
 
-static void rb_auto_playlist_source_songs_sort_order_changed_cb (RBEntryView *view, 
-								 RBAutoPlaylistSource *source); 
+static void rb_auto_playlist_source_songs_sort_order_changed_cb (RBEntryView *view,
+								 RBAutoPlaylistSource *source);
 static void rb_auto_playlist_source_do_query (RBAutoPlaylistSource *source,
 					      gboolean subset);
 
@@ -75,8 +75,6 @@ static GtkRadioActionEntry rb_auto_playlist_source_radio_actions [] =
 	{ "AutoPlaylistSearchTitles", NULL, N_("Titles"), NULL, N_("Search titles"), 3 }
 };
 
-
-
 #define AUTO_PLAYLIST_SOURCE_POPUP_PATH "/AutoPlaylistSourcePopup"
 
 typedef struct _RBAutoPlaylistSourcePrivate RBAutoPlaylistSourcePrivate;
@@ -102,14 +100,14 @@ struct _RBAutoPlaylistSourcePrivate
 	RhythmDBPropType search_prop;
 };
 
+static gpointer playlist_pixbuf = NULL;
+
 G_DEFINE_TYPE (RBAutoPlaylistSource, rb_auto_playlist_source, RB_TYPE_PLAYLIST_SOURCE)
 #define RB_AUTO_PLAYLIST_SOURCE_GET_PRIVATE(object) (G_TYPE_INSTANCE_GET_PRIVATE ((object), RB_TYPE_AUTO_PLAYLIST_SOURCE, RBAutoPlaylistSourcePrivate))
 
 static void
 rb_auto_playlist_source_class_init (RBAutoPlaylistSourceClass *klass)
 {
-	gint size;
-
 	GObjectClass *object_class = G_OBJECT_CLASS (klass);
 	RBSourceClass *source_class = RB_SOURCE_CLASS (klass);
 	RBPlaylistSourceClass *playlist_class = RB_PLAYLIST_SOURCE_CLASS (klass);
@@ -130,22 +128,34 @@ rb_auto_playlist_source_class_init (RBAutoPlaylistSourceClass *klass)
 	source_class->impl_get_search_actions = impl_get_search_actions;
 
 	playlist_class->impl_save_contents_to_xml = impl_save_contents_to_xml;
-	
-	gtk_icon_size_lookup (GTK_ICON_SIZE_LARGE_TOOLBAR, &size, NULL);
-	klass->pixbuf = gtk_icon_theme_load_icon (gtk_icon_theme_get_default (),
-						  GNOME_MEDIA_AUTO_PLAYLIST,
-						  size,
-						  0, NULL);
-	
+
 	g_type_class_add_private (klass, sizeof (RBAutoPlaylistSourcePrivate));
 }
 
 static void
 rb_auto_playlist_source_init (RBAutoPlaylistSource *source)
 {
-	RBAutoPlaylistSourceClass *klass = RB_AUTO_PLAYLIST_SOURCE_GET_CLASS (source);
+	if (playlist_pixbuf == NULL) {
+		gint size;
+
+		gtk_icon_size_lookup (GTK_ICON_SIZE_LARGE_TOOLBAR, &size, NULL);
+		playlist_pixbuf = gtk_icon_theme_load_icon (gtk_icon_theme_get_default (),
+							    GNOME_MEDIA_AUTO_PLAYLIST,
+							    size,
+							    0, NULL);
+		if (playlist_pixbuf) {
+			g_object_add_weak_pointer (playlist_pixbuf,
+						   (gpointer *) &playlist_pixbuf);
+
+			rb_source_set_pixbuf (RB_SOURCE (source), playlist_pixbuf);
+
+			/* drop the initial reference to the icon */
+			g_object_unref (playlist_pixbuf);
+		}
+	} else {
+		rb_source_set_pixbuf (RB_SOURCE (source), playlist_pixbuf);
+	}
 
-	rb_source_set_pixbuf (RB_SOURCE (source), klass->pixbuf);
 }
 
 static void
@@ -153,14 +163,21 @@ rb_auto_playlist_source_finalize (GObject *object)
 {
 	RBAutoPlaylistSourcePrivate *priv = RB_AUTO_PLAYLIST_SOURCE_GET_PRIVATE (object);
 
-	if (priv->cached_all_query)
+	if (priv->action_group != NULL) {
+		g_object_unref (priv->action_group);
+	}
+
+	if (priv->cached_all_query) {
 		g_object_unref (G_OBJECT (priv->cached_all_query));
+	}
 
-	if (priv->query)
+	if (priv->query) {
 		rhythmdb_query_free (priv->query);
+	}
 
-	if (priv->limit_value)
+	if (priv->limit_value) {
 		g_value_array_free (priv->limit_value);
+	}
 
 	g_free (priv->search_text);
 
@@ -187,18 +204,18 @@ rb_auto_playlist_source_constructor (GType type, guint n_construct_properties,
 	g_object_get (RB_PLAYLIST_SOURCE (source), "entry-type", &entry_type, NULL);
 	priv->browser = rb_library_browser_new (rb_playlist_source_get_db (RB_PLAYLIST_SOURCE (source)),
 						entry_type);
+	g_boxed_free (RHYTHMDB_TYPE_ENTRY_TYPE, entry_type);
 	gtk_paned_pack1 (GTK_PANED (priv->paned), GTK_WIDGET (priv->browser), TRUE, FALSE);
 	g_signal_connect_object (G_OBJECT (priv->browser), "notify::output-model",
 				 G_CALLBACK (rb_auto_playlist_source_browser_changed_cb),
 				 source, 0);
 
-
 	songs = rb_source_get_entry_view (RB_SOURCE (source));
 	g_signal_connect_object (G_OBJECT (songs), "sort-order-changed",
 				 G_CALLBACK (rb_auto_playlist_source_songs_sort_order_changed_cb),
 				 source, 0);
 
-	g_object_get (G_OBJECT (source), "shell", &shell, NULL);
+	g_object_get (source, "shell", &shell, NULL);
 	priv->action_group = _rb_source_register_action_group (RB_SOURCE (source),
 							       "AutoPlaylistActions",
 							       NULL, 0,
@@ -210,15 +227,15 @@ rb_auto_playlist_source_constructor (GType type, guint n_construct_properties,
 					    (GCallback)search_action_changed,
 					    shell);
 	priv->search_prop = RHYTHMDB_PROP_SEARCH_MATCH;
-	g_object_unref (G_OBJECT (shell));
-
+	g_object_unref (shell);
 
 	/* reparent the entry view */
-	g_object_ref (G_OBJECT (songs));
+	g_object_ref (songs);
 	gtk_container_remove (GTK_CONTAINER (source), GTK_WIDGET (songs));
 	gtk_paned_pack2 (GTK_PANED (priv->paned), GTK_WIDGET (songs), TRUE, FALSE);
 	gtk_container_add (GTK_CONTAINER (source), priv->paned);
-	
+	g_object_unref (songs);
+
 	gtk_widget_show_all (GTK_WIDGET (source));
 
 	return G_OBJECT (source);
@@ -239,11 +256,10 @@ rb_auto_playlist_source_new (RBShell *shell, const char *name, gboolean local)
 					NULL));
 }
 
-RBSource *	
+RBSource *
 rb_auto_playlist_source_new_from_xml (RBShell *shell, xmlNodePtr node)
 {
 	RBAutoPlaylistSource *source = RB_AUTO_PLAYLIST_SOURCE (rb_auto_playlist_source_new (shell, NULL, TRUE));
-
 	xmlNodePtr child;
 	xmlChar *tmp;
 	GPtrArray *query;
@@ -257,7 +273,7 @@ rb_auto_playlist_source_new_from_xml (RBShell *shell, xmlNodePtr node)
 	while (xmlNodeIsText (child))
 		child = child->next;
 
-	query = rhythmdb_query_deserialize (rb_playlist_source_get_db (RB_PLAYLIST_SOURCE (source)), 
+	query = rhythmdb_query_deserialize (rb_playlist_source_get_db (RB_PLAYLIST_SOURCE (source)),
 					    child);
 
 	limit_value = g_value_array_new (0);
@@ -329,6 +345,7 @@ rb_auto_playlist_source_new_from_xml (RBShell *shell, xmlNodePtr node)
 					   sort_direction);
 	g_free (sort_key);
 	g_value_array_free (limit_value);
+	rhythmdb_query_free (query);
 
 	return RB_SOURCE (source);
 }
@@ -345,7 +362,7 @@ impl_reset_filters (RBSource *source)
 {
 	RBAutoPlaylistSourcePrivate *priv = RB_AUTO_PLAYLIST_SOURCE_GET_PRIVATE (source);
 	gboolean changed = FALSE;
-	
+
 	if (rb_library_browser_reset (priv->browser))
 		changed = TRUE;
 
@@ -369,7 +386,7 @@ impl_search (RBSource *source, const char *search_text)
 
 	if (search_text != NULL && search_text[0] == '\0')
 		search_text = NULL;
-	
+
 	if (search_text == NULL && priv->search_text == NULL)
 		return;
 	if (search_text != NULL && priv->search_text != NULL
@@ -388,7 +405,7 @@ impl_search (RBSource *source, const char *search_text)
 			subset = (g_str_has_prefix (priv->search_text, old_search_text));
 	}
 	g_free (old_search_text);
-	
+
 	/* we can only do subset searches once the original query is complete */
 	if (priv->query_active && subset) {
 		rb_debug ("deferring search for \"%s\" until query completion", debug_search_text);
@@ -441,7 +458,7 @@ static gboolean
 impl_receive_drag (RBSource *asource, GtkSelectionData *data)
 {
 	RBAutoPlaylistSource *source = RB_AUTO_PLAYLIST_SOURCE (asource);
-        
+
 	GPtrArray *subquery = NULL;
 	gchar **names;
 	guint propid;
@@ -454,16 +471,17 @@ impl_receive_drag (RBSource *asource, GtkSelectionData *data)
 
 	names = g_strsplit ((char *)data->data, "\r\n", 0);
 	propid = rb_auto_playlist_source_drag_atom_to_prop (data->type);
-	g_object_get (G_OBJECT (asource), "db", &db, NULL);
 
-	for (i=0; names[i]; i++) {
-		if (subquery == NULL) 
+	g_object_get (asource, "db", &db, NULL);
+
+	for (i = 0; names[i]; i++) {
+		if (subquery == NULL) {
 			subquery = rhythmdb_query_parse (db,
 							 RHYTHMDB_QUERY_PROP_EQUALS,
 							 propid,
 							 names[i],
 							 RHYTHMDB_QUERY_END);
-		else
+		} else {
 			rhythmdb_query_append (db,
 					       subquery,
 					       RHYTHMDB_QUERY_DISJUNCTION,
@@ -471,15 +489,16 @@ impl_receive_drag (RBSource *asource, GtkSelectionData *data)
 					       propid,
 					       names[i],
 					       RHYTHMDB_QUERY_END);
+		}
 	}
 
 	g_strfreev (names);
 
-	if (subquery) {
+	if (subquery != NULL) {
 		RhythmDBEntryType qtype;
 		GPtrArray *query;
-		
-		g_object_get (G_OBJECT (source), "entry-type", &qtype, NULL);
+
+		g_object_get (source, "entry-type", &qtype, NULL);
 		if (qtype == RHYTHMDB_ENTRY_TYPE_INVALID)
 			qtype = RHYTHMDB_ENTRY_TYPE_SONG;
 
@@ -493,9 +512,12 @@ impl_receive_drag (RBSource *asource, GtkSelectionData *data)
 		rb_auto_playlist_source_set_query (RB_AUTO_PLAYLIST_SOURCE (source), query,
 						   RHYTHMDB_QUERY_MODEL_LIMIT_NONE, NULL,
 						   NULL, 0);
+
+		rhythmdb_query_free (subquery);
+		rhythmdb_query_free (query);
 	}
 
-	g_object_unref (G_OBJECT (db));
+	g_object_unref (db);
 
 	return TRUE;
 }
@@ -524,28 +546,31 @@ _save_write_uint64 (xmlNodePtr node, GValueArray *limit_value, const xmlChar *ke
 	g_free (str);
 }
 
-
-static void 
-impl_save_contents_to_xml (RBPlaylistSource *psource, xmlNodePtr node)
+static void
+impl_save_contents_to_xml (RBPlaylistSource *psource,
+			   xmlNodePtr node)
 {
 	GPtrArray *query;
 	RhythmDBQueryModelLimitType limit_type;
 	GValueArray *limit_value = NULL;
-	const gchar *sort_key;
+	char *sort_key;
 	gint sort_direction;
 	RBAutoPlaylistSource *source = RB_AUTO_PLAYLIST_SOURCE (psource);
 
 	xmlSetProp (node, RB_PLAYLIST_TYPE, RB_PLAYLIST_AUTOMATIC);
 
+	sort_key = NULL;
 	rb_auto_playlist_source_get_query (source,
-					    &query,
-					    &limit_type, &limit_value,
-					    &sort_key, &sort_direction);
+					   &query,
+					   &limit_type,
+					   &limit_value,
+					   &sort_key,
+					   &sort_direction);
 
 	switch (limit_type) {
 	case RHYTHMDB_QUERY_MODEL_LIMIT_NONE:
 		break;
-		
+
 	case RHYTHMDB_QUERY_MODEL_LIMIT_COUNT:
 		_save_write_ulong (node, limit_value, RB_PLAYLIST_LIMIT_COUNT);
 		break;
@@ -563,16 +588,18 @@ impl_save_contents_to_xml (RBPlaylistSource *psource, xmlNodePtr node)
 	}
 
 	if (sort_key && *sort_key) {
-		gchar *temp_str;
+		char *temp_str;
 
 		xmlSetProp (node, RB_PLAYLIST_SORT_KEY, BAD_CAST sort_key);
 		temp_str = g_strdup_printf ("%d", sort_direction);
 		xmlSetProp (node, RB_PLAYLIST_SORT_DIRECTION, BAD_CAST temp_str);
+
 		g_free (temp_str);
 	}
 
 	rhythmdb_query_serialize (rb_playlist_source_get_db (psource), query, node);
 	rhythmdb_query_free (query);
+	g_free (sort_key);
 }
 
 static void
@@ -580,7 +607,7 @@ rb_auto_playlist_source_query_complete_cb (RhythmDBQueryModel *model,
 					   RBAutoPlaylistSource *source)
 {
 	RBAutoPlaylistSourcePrivate *priv = RB_AUTO_PLAYLIST_SOURCE_GET_PRIVATE (source);
-	
+
 	priv->query_active = FALSE;
 	if (priv->search_on_completion) {
 		priv->search_on_completion = FALSE;
@@ -594,12 +621,15 @@ static void
 rb_auto_playlist_source_do_query (RBAutoPlaylistSource *source, gboolean subset)
 {
 	RBAutoPlaylistSourcePrivate *priv = RB_AUTO_PLAYLIST_SOURCE_GET_PRIVATE (source);
-	RhythmDB *db = rb_playlist_source_get_db (RB_PLAYLIST_SOURCE (source));
+	RhythmDB *db;
 	RhythmDBQueryModel *query_model;
 	GPtrArray *query;
 
+	/* this doesn't add a ref */
+	db = rb_playlist_source_get_db (RB_PLAYLIST_SOURCE (source));
+
 	g_assert (priv->cached_all_query);
-	
+
 	if (!priv->search_text) {
 		rb_library_browser_set_model (priv->browser,
 					      priv->cached_all_query,
@@ -614,11 +644,11 @@ rb_auto_playlist_source_do_query (RBAutoPlaylistSource *source, gboolean subset)
 
 	if (subset) {
 		/* just apply the new query to the existing query model */
-		g_object_get (G_OBJECT (priv->browser), "input-model", &query_model, NULL);
+		g_object_get (priv->browser, "input-model", &query_model, NULL);
 		g_assert (query_model != priv->cached_all_query);
-		g_object_set (G_OBJECT (query_model), "query", query, NULL);
+		g_object_set (query_model, "query", query, NULL);
 		rhythmdb_query_model_reapply_query (query_model, FALSE);
-		g_object_unref (G_OBJECT (query_model));
+		g_object_unref (query_model);
 	} else {
 		/* otherwise, we need a new query model */
 		query_model = g_object_new (RHYTHMDB_TYPE_QUERY_MODEL,
@@ -626,6 +656,7 @@ rb_auto_playlist_source_do_query (RBAutoPlaylistSource *source, gboolean subset)
 					    "limit-type", priv->limit_type,
 					    "limit-value", priv->limit_value,
 					    NULL);
+		rhythmdb_query_model_chain (query_model, priv->cached_all_query, FALSE);
 		rb_library_browser_set_model (priv->browser, query_model, TRUE);
 
 		priv->query_active = TRUE;
@@ -633,11 +664,12 @@ rb_auto_playlist_source_do_query (RBAutoPlaylistSource *source, gboolean subset)
 		g_signal_connect_object (G_OBJECT (query_model),
 					 "complete", G_CALLBACK (rb_auto_playlist_source_query_complete_cb),
 					 source, 0);
-		rhythmdb_do_full_query_async_parsed (db, 
-						     RHYTHMDB_QUERY_RESULTS (query_model), 
+		rhythmdb_do_full_query_async_parsed (db,
+						     RHYTHMDB_QUERY_RESULTS (query_model),
 						     query);
 		g_object_unref (G_OBJECT (query_model));
 	}
+
 	rhythmdb_query_free (query);
 }
 
@@ -654,10 +686,17 @@ rb_auto_playlist_source_set_query (RBAutoPlaylistSource *source,
 	RBEntryView *songs = rb_source_get_entry_view (RB_SOURCE (source));
 
 	priv->query_resetting = TRUE;
-	if (priv->query)
+	if (priv->query) {
 		rhythmdb_query_free (priv->query);
-	if (priv->cached_all_query)
+	}
+
+	if (priv->cached_all_query) {
 		g_object_unref (G_OBJECT (priv->cached_all_query));
+	}
+
+	if (priv->limit_value) {
+		g_value_array_free (priv->limit_value);
+	}
 
 	/* playlists that aren't limited, with a particular sort order, are user-orderable */
 	rb_entry_view_set_columns_clickable (songs, (limit_type == RHYTHMDB_QUERY_MODEL_LIMIT_NONE));
@@ -665,34 +704,36 @@ rb_auto_playlist_source_set_query (RBAutoPlaylistSource *source,
 
 	priv->query = rhythmdb_query_copy (query);
 	priv->limit_type = limit_type;
-	if (priv->limit_value)
-		g_value_array_free (priv->limit_value);
 	priv->limit_value = limit_value ? g_value_array_copy (limit_value) : NULL;
-	
-	priv->cached_all_query = g_object_new (RHYTHMDB_TYPE_QUERY_MODEL,
-					      "db", db,
-					      "limit-type", priv->limit_type,
-					      "limit-value", priv->limit_value,
-					      NULL);
-	rb_library_browser_set_model (priv->browser, priv->cached_all_query, TRUE);
-	rhythmdb_do_full_query_async_parsed (db, 
-					     RHYTHMDB_QUERY_RESULTS (priv->cached_all_query), 
-					     query);
 
+	priv->cached_all_query = g_object_new (RHYTHMDB_TYPE_QUERY_MODEL,
+					       "db", db,
+					       "limit-type", priv->limit_type,
+					       "limit-value", priv->limit_value,
+					       NULL);
+	rb_library_browser_set_model (priv->browser, priv->cached_all_query, TRUE);
+	rhythmdb_do_full_query_async_parsed (db,
+					     RHYTHMDB_QUERY_RESULTS (priv->cached_all_query),
+					     priv->query);
 
 	priv->query_resetting = FALSE;
 }
 
 void
 rb_auto_playlist_source_get_query (RBAutoPlaylistSource *source,
-				    GPtrArray **query,
-				    RhythmDBQueryModelLimitType *limit_type,
-				    GValueArray **limit_value,
-				    const char **sort_key,
-				    gint *sort_direction)
+				   GPtrArray **query,
+				   RhythmDBQueryModelLimitType *limit_type,
+				   GValueArray **limit_value,
+				   char **sort_key,
+				   gint *sort_direction)
 {
-	RBAutoPlaylistSourcePrivate *priv = RB_AUTO_PLAYLIST_SOURCE_GET_PRIVATE (source);
-	RBEntryView *songs = rb_source_get_entry_view (RB_SOURCE (source));
+	RBAutoPlaylistSourcePrivate *priv;
+	RBEntryView *songs;
+
+ 	g_return_if_fail (RB_IS_AUTO_PLAYLIST_SOURCE (source));
+
+	priv = RB_AUTO_PLAYLIST_SOURCE_GET_PRIVATE (source);
+	songs = rb_source_get_entry_view (RB_SOURCE (source));
 
 	*query = rhythmdb_query_copy (priv->query);
 	*limit_type = priv->limit_type;
@@ -721,12 +762,12 @@ rb_auto_playlist_source_browser_changed_cb (RBLibraryBrowser *browser,
 {
 	RBEntryView *songs = rb_source_get_entry_view (RB_SOURCE (source));
 	RhythmDBQueryModel *query_model;
-	
-	g_object_get (G_OBJECT (browser), "output-model", &query_model, NULL);
+
+	g_object_get (browser, "output-model", &query_model, NULL);
 	rb_entry_view_set_model (songs, query_model);
 	rb_playlist_source_set_query_model (RB_PLAYLIST_SOURCE (source), query_model);
-	g_object_unref (G_OBJECT (query_model));
-	
+	g_object_unref (query_model);
+
 	rb_source_notify_filter_changed (RB_SOURCE (source));
 }
 
@@ -754,7 +795,7 @@ search_action_to_prop (GtkAction *action)
 	if (name == NULL) {
 		prop = RHYTHMDB_PROP_SEARCH_MATCH;
 	} else if (strcmp (name, "AutoPlaylistSearchAll") == 0) {
-		prop = RHYTHMDB_PROP_SEARCH_MATCH;		
+		prop = RHYTHMDB_PROP_SEARCH_MATCH;
 	} else if (strcmp (name, "AutoPlaylistSearchArtists") == 0) {
 		prop = RHYTHMDB_PROP_ARTIST_FOLDED;
 	} else if (strcmp (name, "AutoPlaylistSearchAlbums") == 0) {
@@ -777,7 +818,8 @@ search_action_changed (GtkRadioAction  *action,
 	gboolean active;
 	RBAutoPlaylistSource *source;
 
-	g_object_get (G_OBJECT (shell), "selected-source", &source, NULL);
+	g_object_get (shell, "selected-source", &source, NULL);
+
 	priv = RB_AUTO_PLAYLIST_SOURCE_GET_PRIVATE (source);
 
 	active = gtk_toggle_action_get_active (GTK_TOGGLE_ACTION (current));
@@ -788,4 +830,8 @@ search_action_changed (GtkRadioAction  *action,
 		rb_auto_playlist_source_do_query (source, FALSE);
 		rb_source_notify_filter_changed (RB_SOURCE (source));
 	}
+
+	if (source != NULL) {
+		g_object_unref (source);
+	}
 }
