@@ -15,7 +15,7 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA.
  *
  */
 
@@ -47,23 +47,30 @@ typedef struct {
 	RBBrowserSourceClass parent;
 } RBDAAPSourceClass;
 
-RBSource * 
+RBSource *
 rb_daap_sources_init (RBShell *shell);
 
-void 
+void
 rb_daap_sources_shutdown (RBShell *shell);
 
-GType 
+GType
 rb_daap_source_get_type (void);
 
-RBDAAPSource *	
+RBDAAPSource *
 rb_daap_source_find_for_uri (const gchar *uri);
 
+#ifdef HAVE_GSTREAMER_0_8
 gchar *
-rb_daap_source_get_headers (RBDAAPSource *source, 
-			    const gchar *uri, 
+rb_daap_source_get_headers (RBDAAPSource *source,
+			    const gchar *uri,
 			    glong time,
 			    gint64 *bytes);
+#else
+gchar *
+rb_daap_source_get_headers (RBDAAPSource *source,
+			    const gchar *uri, 
+			    gint64 bytes);
+#endif
 
 G_END_DECLS
 
