@@ -21,7 +21,7 @@
  *
  */
 
-#include <config.h>
+#include "config.h"
 
 #include <unistd.h>
 #include <string.h>
@@ -32,7 +32,6 @@
 #include <libgnomevfs/gnome-vfs-uri.h>
 #include <totem-pl-parser.h>
 
-#include "rb-stock-icons.h"
 #include "rb-entry-view.h"
 #include "rb-search-entry.h"
 #include "rb-file-helpers.h"
@@ -40,7 +39,6 @@
 #include "rb-dialog.h"
 #include "rb-util.h"
 #include "rb-playlist-source.h"
-#include "rb-playlist-source-recorder.h"
 #include "rb-debug.h"
 #include "eel-gconf-extensions.h"
 #include "rb-song-info.h"
@@ -51,25 +49,25 @@
 
 static void rb_playlist_source_class_init (RBPlaylistSourceClass *klass);
 static void rb_playlist_source_init (RBPlaylistSource *source);
-static GObject *rb_playlist_source_constructor (GType type, guint n_construct_properties,
+static GObject *rb_playlist_source_constructor (GType type,
+						guint n_construct_properties,
 						GObjectConstructParam *construct_properties);
 static void rb_playlist_source_dispose (GObject *object);
 static void rb_playlist_source_finalize (GObject *object);
 static void rb_playlist_source_set_property (GObject *object,
-			                  guint prop_id,
-			                  const GValue *value,
-			                  GParamSpec *pspec);
+					     guint prop_id,
+					     const GValue *value,
+					     GParamSpec *pspec);
 static void rb_playlist_source_get_property (GObject *object,
-			                  guint prop_id,
-			                  GValue *value,
-			                  GParamSpec *pspec);
+					     guint prop_id,
+					     GValue *value,
+					     GParamSpec *pspec);
 
 /* source methods */
 static char *impl_get_browser_key (RBSource *source);
 static RBEntryView *impl_get_entry_view (RBSource *source);
 static void impl_song_properties (RBSource *source);
 static gboolean impl_show_popup (RBSource *source);
-static GList *impl_get_ui_actions (RBSource *source);
 
 static void rb_playlist_source_songs_show_popup_cb (RBEntryView *view,
 						    gboolean over_entry,
@@ -83,7 +81,6 @@ static void rb_playlist_source_drop_cb (GtkWidget *widget,
 				     guint time,
 				     gpointer user_data);
 
-
 static void rb_playlist_source_row_deleted (GtkTreeModel *model,
 					    GtkTreePath *path,
 					    RBPlaylistSource *playlist);
@@ -105,13 +102,13 @@ struct RBPlaylistSourcePrivate
 
 	GHashTable *entries;
 
-	RhythmDBEntryType entry_type;
 	RhythmDBQueryModel *model;
 
 	RBEntryView *songs;
 
 	gboolean dirty;
 	gboolean is_local;
+	gboolean dispose_has_run;
 
 	char *title;
 };
@@ -124,14 +121,12 @@ enum
 	PROP_DB,
 	PROP_DIRTY,
 	PROP_LOCAL,
-	PROP_ENTRY_TYPE
 };
 
 static const GtkTargetEntry target_uri [] = { { "text/uri-list", 0, 0 } };
 
 G_DEFINE_ABSTRACT_TYPE (RBPlaylistSource, rb_playlist_source, RB_TYPE_SOURCE);
 
-
 static void
 rb_playlist_source_class_init (RBPlaylistSourceClass *klass)
 {
@@ -156,9 +151,7 @@ rb_playlist_source_class_init (RBPlaylistSourceClass *klass)
 	source_class->impl_can_move_to_trash = (RBSourceFeatureFunc) rb_true_function;
 	source_class->impl_song_properties = impl_song_properties;
 	source_class->impl_can_pause = (RBSourceFeatureFunc) rb_true_function;
-	source_class->impl_have_url = (RBSourceFeatureFunc) rb_false_function;
 	source_class->impl_show_popup = impl_show_popup;
-	source_class->impl_get_ui_actions = impl_get_ui_actions;
 
 	klass->impl_show_entry_view_popup = default_show_entry_view_popup;
 
@@ -184,13 +177,6 @@ rb_playlist_source_class_init (RBPlaylistSourceClass *klass)
 							       TRUE,
 							       G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
 
-	g_object_class_install_property (object_class,
-					 PROP_ENTRY_TYPE,
-					 g_param_spec_pointer ("entry-type",
-							       "Entry type",
-							       "Type of the entries which should be displayed by this playlist",
-							       G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
-
 	g_type_class_add_private (klass, sizeof (RBPlaylistSourcePrivate));
 }
 
@@ -198,48 +184,79 @@ static void
 rb_playlist_source_init (RBPlaylistSource *source)
 {
 	source->priv = RB_PLAYLIST_SOURCE_GET_PRIVATE (source);
+}
+
+static void
+rb_playlist_source_set_db (RBPlaylistSource *source,
+			   RhythmDB         *db)
+{
+	if (source->priv->db != NULL) {
+		g_signal_handlers_disconnect_by_func (source->priv->db,
+						      rb_playlist_source_entry_added_cb,
+						      source);
+		g_object_unref (source->priv->db);
+
+	}
+
+	source->priv->db = db;
+
+	if (source->priv->db != NULL) {
+		g_object_ref (source->priv->db);
+		g_signal_connect_object (G_OBJECT (source->priv->db), "entry_added",
+					 G_CALLBACK (rb_playlist_source_entry_added_cb),
+					 source, 0);
+	}
 
 }
 
 static GObject *
-rb_playlist_source_constructor (GType type, guint n_construct_properties,
+rb_playlist_source_constructor (GType type,
+				guint n_construct_properties,
 				GObjectConstructParam *construct_properties)
 {
 	GObject *shell_player;
 	RBPlaylistSource *source;
 	RBPlaylistSourceClass *klass;
 	RBShell *shell;
+	RhythmDB *db;
+	RhythmDBQueryModel *query_model;
 
 	klass = RB_PLAYLIST_SOURCE_CLASS (g_type_class_peek (RB_TYPE_PLAYLIST_SOURCE));
 
 	source = RB_PLAYLIST_SOURCE (G_OBJECT_CLASS (rb_playlist_source_parent_class)->
 			constructor (type, n_construct_properties, construct_properties));
 
-	if (source->priv->entry_type == NULL)
-		source->priv->entry_type = RHYTHMDB_ENTRY_TYPE_INVALID;
-
-	g_object_get (G_OBJECT (source), "shell", &shell, NULL);
-	g_object_get (G_OBJECT (shell), "db", &source->priv->db, NULL);
+	g_object_get (source, "shell", &shell, NULL);
+	g_object_get (shell, "db", &db, NULL);
 	shell_player = rb_shell_get_player (shell);
-	g_object_unref (G_OBJECT (shell));
+	rb_playlist_source_set_db (source, db);
+	g_object_unref (db);
+	g_object_unref (shell);
 
-	source->priv->entries = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, NULL);
+	source->priv->entries = g_hash_table_new_full (rb_refstring_hash, rb_refstring_equal,
+						       (GDestroyNotify)rb_refstring_unref, NULL);
 
-	g_signal_connect_object (G_OBJECT (source->priv->db), "entry_added",
-				 G_CALLBACK (rb_playlist_source_entry_added_cb),
-				 source, 0);
-
-	source->priv->songs = rb_entry_view_new (source->priv->db, shell_player, 
+	source->priv->songs = rb_entry_view_new (source->priv->db,
+						 shell_player,
 					 	 NULL, TRUE, TRUE);
 
-	rb_playlist_source_set_query_model (source, rhythmdb_query_model_new_empty (source->priv->db));
+	query_model = rhythmdb_query_model_new_empty (source->priv->db);
+	rb_playlist_source_set_query_model (source, query_model);
+	g_object_unref (query_model);
 
 	{
-		const char *title = _("Trac_k");
+		const char *title = "";
 		const char *strings[3] = {0};
 
 		GtkTreeViewColumn *column = gtk_tree_view_column_new ();
 		GtkCellRenderer *renderer = gtk_cell_renderer_text_new ();
+
+		g_object_set(renderer,
+			     "style", PANGO_STYLE_OBLIQUE,
+			     "weight", PANGO_WEIGHT_LIGHT,
+			     "xalign", 1.0,
+			     NULL);
+
 		gtk_tree_view_column_pack_start (column, renderer, TRUE);
 
 		gtk_tree_view_column_set_resizable (column, TRUE);
@@ -253,10 +270,11 @@ rb_playlist_source_constructor (GType type, guint n_construct_properties,
 							 (GtkTreeCellDataFunc)
 							 rb_playlist_source_track_cell_data_func,
 							 source, NULL);
-		rb_entry_view_append_column_custom (source->priv->songs, column, title,
-						    "PlaylistTrack", NULL, 0);
+		rb_entry_view_insert_column_custom (source->priv->songs, column, title,
+						    "PlaylistTrack", NULL, 0, 0);
 	}
 
+	rb_entry_view_append_column (source->priv->songs, RB_ENTRY_VIEW_COL_TRACK_NUMBER, FALSE);
 	rb_entry_view_append_column (source->priv->songs, RB_ENTRY_VIEW_COL_TITLE, TRUE);
 	rb_entry_view_append_column (source->priv->songs, RB_ENTRY_VIEW_COL_GENRE, FALSE);
 	rb_entry_view_append_column (source->priv->songs, RB_ENTRY_VIEW_COL_ARTIST, FALSE);
@@ -273,7 +291,7 @@ rb_playlist_source_constructor (GType type, guint n_construct_properties,
 	rb_playlist_source_setup_entry_view (source, source->priv->songs);
 
 	gtk_container_add (GTK_CONTAINER (source), GTK_WIDGET (source->priv->songs));
-		
+
 	gtk_widget_show_all (GTK_WIDGET (source));
 
 	return G_OBJECT (source);
@@ -283,11 +301,26 @@ static void
 rb_playlist_source_dispose (GObject *object)
 {
 	RBPlaylistSource *source = RB_PLAYLIST_SOURCE (object);
-	if (source->priv->db) {
+
+	if (source->priv->dispose_has_run) {
+		/* If dispose did already run, return. */
+		rb_debug ("Dispose has already run for playlist source %p", object);
+		return;
+	}
+	/* Make sure dispose does not run twice. */
+	source->priv->dispose_has_run = TRUE;
+
+	rb_debug ("Disposing playlist source %p", source);
+
+	if (source->priv->db != NULL) {
 		g_object_unref (source->priv->db);
 		source->priv->db = NULL;
 	}
 
+	if (source->priv->model != NULL) {
+		g_object_unref (source->priv->model);
+	}
+
 	G_OBJECT_CLASS (rb_playlist_source_parent_class)->dispose (object);
 }
 
@@ -302,6 +335,8 @@ rb_playlist_source_finalize (GObject *object)
 	source = RB_PLAYLIST_SOURCE (object);
 	g_return_if_fail (source->priv != NULL);
 
+	rb_debug ("Finalizing playlist source %p", source);
+
 	g_hash_table_destroy (source->priv->entries);
 
 	g_free (source->priv->title);
@@ -312,20 +347,16 @@ rb_playlist_source_finalize (GObject *object)
 
 static void
 rb_playlist_source_set_property (GObject *object,
-			      guint prop_id,
-			      const GValue *value,
-			      GParamSpec *pspec)
+				 guint prop_id,
+				 const GValue *value,
+				 GParamSpec *pspec)
 {
 	RBPlaylistSource *source = RB_PLAYLIST_SOURCE (object);
 
-	switch (prop_id)
-	{
+	switch (prop_id) {
 	case PROP_LOCAL:
 		source->priv->is_local = g_value_get_boolean (value);
 		break;
-	case PROP_ENTRY_TYPE:
-		source->priv->entry_type = g_value_get_pointer (value);
-		break;
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
 		break;
@@ -334,14 +365,13 @@ rb_playlist_source_set_property (GObject *object,
 
 static void
 rb_playlist_source_get_property (GObject *object,
-			      guint prop_id,
-			      GValue *value,
-			      GParamSpec *pspec)
+				 guint prop_id,
+				 GValue *value,
+				 GParamSpec *pspec)
 {
 	RBPlaylistSource *source = RB_PLAYLIST_SOURCE (object);
 
-	switch (prop_id)
-	{
+	switch (prop_id) {
 	case PROP_DB:
 		g_value_set_object (value, source->priv->db);
 		break;
@@ -351,9 +381,6 @@ rb_playlist_source_get_property (GObject *object,
 	case PROP_LOCAL:
 		g_value_set_boolean (value, source->priv->is_local);
 		break;
-	case PROP_ENTRY_TYPE:
-		g_value_set_pointer (value, source->priv->entry_type);
-		break;
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
 		break;
@@ -379,7 +406,6 @@ rb_playlist_source_songs_show_popup_cb (RBEntryView *view,
 		klass->impl_show_entry_view_popup (source, view, over_entry);
 }
 
-
 static char *
 impl_get_browser_key (RBSource *source)
 {
@@ -454,8 +480,16 @@ playlist_iter_func (GtkTreeModel *model,
 
 	gtk_tree_model_get (model, iter, 0, &entry, -1);
 
-	*uri = rhythmdb_entry_dup_string (entry, RHYTHMDB_PROP_LOCATION);
-	*title = rhythmdb_entry_dup_string (entry, RHYTHMDB_PROP_TITLE);
+	if (uri != NULL) {
+		*uri = rhythmdb_entry_dup_string (entry, RHYTHMDB_PROP_LOCATION);
+	}
+	if (title != NULL) {
+		*title = rhythmdb_entry_dup_string (entry, RHYTHMDB_PROP_TITLE);
+	}
+
+	if (entry != NULL) {
+		rhythmdb_entry_unref (entry);
+	}
 }
 #else
 static void
@@ -470,14 +504,24 @@ playlist_iter_func (GtkTreeModel *model,
 
 	gtk_tree_model_get (model, iter, 0, &entry, -1);
 
-	*uri = rhythmdb_entry_dup_string (entry, RHYTHMDB_PROP_LOCATION);
-	*title = rhythmdb_entry_dup_string (entry, RHYTHMDB_PROP_TITLE);
-	*custom_title = FALSE;
+	if (uri != NULL) {
+		*uri = rhythmdb_entry_dup_string (entry, RHYTHMDB_PROP_LOCATION);
+	}
+	if (title != NULL) {
+		*title = rhythmdb_entry_dup_string (entry, RHYTHMDB_PROP_TITLE);
+	}
+	if (custom_title != NULL) {
+		*custom_title = FALSE;
+	}
+
+	if (entry != NULL) {
+		rhythmdb_entry_unref (entry);
+	}
 }
 #endif /* TOTEM_PL_PARSER_CHECK_VERSION */
 
 void
-rb_playlist_source_save_playlist (RBPlaylistSource *source, 
+rb_playlist_source_save_playlist (RBPlaylistSource *source,
 				  const char *uri,
 				  gboolean m3u_format)
 {
@@ -485,13 +529,15 @@ rb_playlist_source_save_playlist (RBPlaylistSource *source,
 	GError *error = NULL;
 	char *name;
 
+	g_return_if_fail (RB_IS_PLAYLIST_SOURCE (source));
+
 	rb_debug ("saving playlist");
 	playlist = totem_pl_parser_new ();
 
-	g_object_get (G_OBJECT (source), "name", &name, NULL);
+	g_object_get (source, "name", &name, NULL);
 
 	totem_pl_parser_write_with_title (playlist, GTK_TREE_MODEL (source->priv->model),
-					  playlist_iter_func, uri, name, 
+					  playlist_iter_func, uri, name,
 					  m3u_format ? TOTEM_PL_PARSER_M3U : TOTEM_PL_PARSER_PLS,
 					  NULL, &error);
 	g_free (name);
@@ -500,70 +546,6 @@ rb_playlist_source_save_playlist (RBPlaylistSource *source,
 				 "%s", error->message);
 }
 
-static gboolean
-burn_playlist_iter_func (GtkTreeModel *model, GtkTreeIter *iter, char **uri, char **artist, char **title, gulong *duration)
-{
-	RhythmDBEntry *entry;
-
-	gtk_tree_model_get (model, iter, 0, &entry, -1);
-
-	*uri = rhythmdb_entry_dup_string (entry, RHYTHMDB_PROP_LOCATION);
-	*title = rhythmdb_entry_dup_string (entry, RHYTHMDB_PROP_TITLE);
-	*artist = rhythmdb_entry_dup_string (entry, RHYTHMDB_PROP_ARTIST);
-	*duration = rhythmdb_entry_get_ulong (entry, RHYTHMDB_PROP_DURATION);
-
-	return TRUE;
-}
-
-void
-rb_playlist_source_burn_playlist (RBPlaylistSource *source)
-{
-	GtkWidget *recorder;
-	GtkWidget *parent;
-	char *name;
-	RBShell *shell;
-	gboolean res;
-	GError *error;
-
-	/* don't burn if the playlist is empty */
-	if (gtk_tree_model_iter_n_children (GTK_TREE_MODEL (source->priv->model), NULL) == 0)
-		return;
-
-	rb_debug ("burning playlist");
-
-	g_object_get (source, "name", &name, "shell", &shell, NULL);
-
-	parent = gtk_widget_get_toplevel (GTK_WIDGET (source));
-	recorder = rb_playlist_source_recorder_new (parent,
-						    shell,
-						    name);
-	g_object_unref (shell);
-	g_free (name);
-
-	error = NULL;
-	res = rb_playlist_source_recorder_add_from_model (RB_PLAYLIST_SOURCE_RECORDER (recorder),
-							  GTK_TREE_MODEL (source->priv->model),
-							  burn_playlist_iter_func,
-							  &error);
-	if (! res) {
-		rb_error_dialog (GTK_WINDOW (parent),
-				 _("Unable to create audio CD"),
-				 "%s", error->message);
-		g_error_free (error);
-
-		gtk_widget_destroy (recorder);
-
-		return;
-	}
-
-        g_signal_connect (recorder,
-			  "response",
-			  G_CALLBACK (gtk_widget_destroy),
-			  NULL);
-
-	gtk_widget_show (recorder);
-}
-
 /* Adapted from yelp-toc-pager.c */
 static xmlChar *
 xml_get_and_trim_names (xmlNodePtr node)
@@ -653,10 +635,13 @@ rb_playlist_source_new_from_xml	(RBShell *shell,
 	xmlChar *tmp;
 	xmlChar *name;
 
+	g_return_val_if_fail (RB_IS_SHELL (shell), NULL);
+
 	/* Try to get name from XML and remove translated names */
 	name = get_playlist_name_from_xml (node);
 
 	tmp = xmlGetProp (node, RB_PLAYLIST_TYPE);
+
 	if (!xmlStrcmp (tmp, RB_PLAYLIST_AUTOMATIC))
 		source = rb_auto_playlist_source_new_from_xml (shell, node);
 	else if (!xmlStrcmp (tmp, RB_PLAYLIST_STATIC))
@@ -666,29 +651,33 @@ rb_playlist_source_new_from_xml	(RBShell *shell,
 
 		g_object_get (shell, "queue-source", &queue, NULL);
 		rb_static_playlist_source_load_from_xml (queue, node);
-		g_object_unref (G_OBJECT (queue));
-		return NULL;
+		g_object_unref (queue);
 	} else {
 		g_warning ("attempting to load playlist '%s' of unknown type '%s'", name, tmp);
 	}
 
-	if (source)
+	if (source != NULL) {
 		g_object_set (G_OBJECT (source), "name", name, NULL);
+	}
 
-	g_free (name);
+	xmlFree (name);
+	xmlFree (tmp);
 
 	return source;
 }
 
 void
-rb_playlist_source_save_to_xml (RBPlaylistSource *source, xmlNodePtr parent_node)
+rb_playlist_source_save_to_xml (RBPlaylistSource *source,
+				xmlNodePtr parent_node)
 {
 	xmlNodePtr node;
 	xmlChar *name;
 	RBPlaylistSourceClass *klass = RB_PLAYLIST_SOURCE_GET_CLASS (source);
 
+	g_return_if_fail (RB_IS_PLAYLIST_SOURCE (source));
+
 	node = xmlNewChild (parent_node, NULL, RB_PLAYLIST_PLAYLIST, NULL);
-	g_object_get (G_OBJECT (source), "name", &name, NULL);
+	g_object_get (source, "name", &name, NULL);
 	xmlSetProp (node, RB_PLAYLIST_NAME, name);
 	g_free (name);
 
@@ -697,41 +686,47 @@ rb_playlist_source_save_to_xml (RBPlaylistSource *source, xmlNodePtr parent_node
 	source->priv->dirty = FALSE;
 }
 
-
 static void
 rb_playlist_source_row_deleted (GtkTreeModel *model,
 				GtkTreePath *path,
 				RBPlaylistSource *source)
 {
 	RhythmDBEntry *entry;
-	const char *location;
+	RBRefString *location;
 
 	entry = rhythmdb_query_model_tree_path_to_entry (RHYTHMDB_QUERY_MODEL (model),
 							 path);
 
-	location = rhythmdb_entry_get_string (entry, RHYTHMDB_PROP_LOCATION);
+	location = rhythmdb_entry_get_refstring (entry, RHYTHMDB_PROP_LOCATION);
 	if (g_hash_table_remove (source->priv->entries, location))
 		source->priv->dirty = TRUE;
+
+	rb_refstring_unref (location);
+	rhythmdb_entry_unref (entry);
 }
 
 static void
-rb_playlist_source_entry_added_cb (RhythmDB *db, 
+rb_playlist_source_entry_added_cb (RhythmDB *db,
 				   RhythmDBEntry *entry,
 				   RBPlaylistSource *source)
 {
-	const char *location;
+	RBRefString *location;
 
-	location = rhythmdb_entry_get_string (entry, RHYTHMDB_PROP_LOCATION);
+	location = rhythmdb_entry_get_refstring (entry, RHYTHMDB_PROP_LOCATION);
 
 	if (g_hash_table_lookup (source->priv->entries, location)) {
 		rhythmdb_query_model_add_entry (source->priv->model, entry, -1);
 		source->priv->dirty = TRUE;
 	}
+
+	rb_refstring_unref (location);
 }
 
 static void
-rb_playlist_source_track_cell_data_func (GtkTreeViewColumn *column, GtkCellRenderer *renderer,
-					 GtkTreeModel *tree_model, GtkTreeIter *iter,
+rb_playlist_source_track_cell_data_func (GtkTreeViewColumn *column,
+					 GtkCellRenderer *renderer,
+					 GtkTreeModel *tree_model,
+					 GtkTreeIter *iter,
 					 RBPlaylistSource *source)
 {
 	char *str;
@@ -752,56 +747,70 @@ void
 rb_playlist_source_setup_entry_view (RBPlaylistSource *source,
 				     RBEntryView *entry_view)
 {
-	g_signal_connect_object (G_OBJECT (entry_view), "show_popup",
+	g_return_if_fail (RB_IS_PLAYLIST_SOURCE (source));
+
+	g_signal_connect_object (entry_view, "show_popup",
 				 G_CALLBACK (rb_playlist_source_songs_show_popup_cb), source, 0);
-	g_signal_connect_object (G_OBJECT (entry_view), "drag_data_received",
+	g_signal_connect_object (entry_view, "drag_data_received",
 				 G_CALLBACK (rb_playlist_source_drop_cb), source, 0);
-	gtk_drag_dest_set (GTK_WIDGET (entry_view), GTK_DEST_DEFAULT_ALL,
-			   target_uri, G_N_ELEMENTS (target_uri), GDK_ACTION_COPY);
+	gtk_drag_dest_set (GTK_WIDGET (entry_view),
+			   GTK_DEST_DEFAULT_ALL,
+			   target_uri,
+			   G_N_ELEMENTS (target_uri),
+			   GDK_ACTION_COPY);
 }
 
 void
 rb_playlist_source_set_query_model (RBPlaylistSource *source,
 				    RhythmDBQueryModel *model)
 {
-	if (source->priv->model) {
+	g_return_if_fail (RB_IS_PLAYLIST_SOURCE (source));
+
+	if (source->priv->model != NULL) {
 		/* if the query model is replaced, the set of entries in
 		 * the playlist will change, so we should mark the playlist dirty.
 		 */
 		source->priv->dirty = TRUE;
-		g_signal_handlers_disconnect_by_func (G_OBJECT (source->priv->model),
+		g_signal_handlers_disconnect_by_func (source->priv->model,
 						      G_CALLBACK (rb_playlist_source_row_deleted),
 						      source);
 		g_object_unref (source->priv->model);
 	}
+
 	source->priv->model = model;
 
-	if (model) {
-		g_object_ref (G_OBJECT (model));
-		g_signal_connect_object (G_OBJECT (source->priv->model), "row_deleted",
+	if (source->priv->model != NULL) {
+		g_object_ref (source->priv->model);
+		g_signal_connect_object (source->priv->model, "row_deleted",
 					 G_CALLBACK (rb_playlist_source_row_deleted), source, 0);
 	}
 
 	rb_entry_view_set_model (source->priv->songs, RHYTHMDB_QUERY_MODEL (source->priv->model));
-	
-	g_object_set (G_OBJECT (source), "query-model", source->priv->model, NULL);
+
+	g_object_set (source, "query-model", source->priv->model, NULL);
 }
 
 RhythmDB *
 rb_playlist_source_get_db (RBPlaylistSource *source)
 {
+	g_return_val_if_fail (RB_IS_PLAYLIST_SOURCE (source), NULL);
+
 	return source->priv->db;
 }
 
 RhythmDBQueryModel *
 rb_playlist_source_get_query_model (RBPlaylistSource *source)
 {
+	g_return_val_if_fail (RB_IS_PLAYLIST_SOURCE (source), NULL);
+
 	return source->priv->model;
 }
 
 void
 rb_playlist_source_mark_dirty (RBPlaylistSource *source)
 {
+	g_return_if_fail (RB_IS_PLAYLIST_SOURCE (source));
+
 	source->priv->dirty = TRUE;
 }
 
@@ -809,30 +818,38 @@ gboolean
 rb_playlist_source_location_in_map (RBPlaylistSource *source,
 				    const char *location)
 {
-	return (g_hash_table_lookup (source->priv->entries, location) != NULL);
+	RBRefString *refstr;
+	gboolean found;
+
+	g_return_val_if_fail (RB_IS_PLAYLIST_SOURCE (source), FALSE);
+
+	refstr = rb_refstring_find (location);
+	if (refstr == NULL) {
+		return FALSE;
+	}
+
+	found = (g_hash_table_lookup (source->priv->entries, refstr) != NULL);
+	rb_refstring_unref (refstr);
+
+	return found;
 }
 
 gboolean
 rb_playlist_source_add_to_map (RBPlaylistSource *source,
 			       const char *location)
 {
-	if (g_hash_table_lookup (source->priv->entries, location)) {
+	RBRefString *refstr;
+
+	g_return_val_if_fail (RB_IS_PLAYLIST_SOURCE (source), FALSE);
+
+	refstr = rb_refstring_new (location);
+	if (g_hash_table_lookup (source->priv->entries, refstr)) {
+		rb_refstring_unref (refstr);
 		return FALSE;
 	}
 
 	g_hash_table_insert (source->priv->entries,
-			     g_strdup (location), GINT_TO_POINTER (1));
+			     refstr, GINT_TO_POINTER (1));
+
 	return TRUE;
 }
-
-static GList*
-impl_get_ui_actions (RBSource *source)
-{
-	GList *actions = NULL;
-
-	actions = g_list_prepend (actions, g_strdup ("MusicPlaylistBurnPlaylist"));
-
-	return actions;
-}
-
-
