@@ -16,7 +16,7 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA.
  *
  */
 
@@ -38,6 +38,21 @@ typedef enum {
 	RB_SOURCE_EOF_NEXT,
 } RBSourceEOFType;
 
+/* groupings in the source list */
+typedef enum {
+	RB_SOURCELIST_GROUP_FIXED,	/* library, iradio, podcast, queue */
+	RB_SOURCELIST_GROUP_PERSISTANT,	/* playlists */
+	RB_SOURCELIST_GROUP_REMOVABLE,	/* ipod, generic audio player, audio CD */
+	RB_SOURCELIST_GROUP_TRANSIENT,	/* DAAP shares */
+
+	RB_SOURCELIST_GROUP_LAST
+} RBSourceListGroup;
+
+GType rb_source_eof_type_get_type (void);
+#define RB_TYPE_SOURCE_EOF_TYPE	(rb_source_eof_type_get_type())
+
+GType rb_sourcelist_group_get_type (void);
+#define RB_TYPE_SOURCELIST_GROUP (rb_sourcelist_group_get_type())
 
 #define RB_TYPE_SOURCE         (rb_source_get_type ())
 #define RB_SOURCE(o)           (G_TYPE_CHECK_INSTANCE_CAST ((o), RB_TYPE_SOURCE, RBSource))
@@ -56,14 +71,12 @@ typedef const char * (*RBSourceStringFunc) (RBSource *source);
 struct _RBSource
 {
 	GtkHBox parent;
-
-	RBSourcePrivate *priv;
 };
 
 struct _RBSourceClass
 {
 	GtkHBoxClass parent;
-	
+
 	/* signals */
 	void (*status_changed)	(RBSource *source);
 	void (*filter_changed)	(RBSource *source);
@@ -71,15 +84,15 @@ struct _RBSourceClass
 	void (*artistalbum_changed)	(RBSource *source);
 
 	/* methods */
-	char *	        (*impl_get_status)	(RBSource *source);
+	void		(*impl_get_status)	(RBSource *source, char **text, char **progress_text, float *progress);
 
 	gboolean	(*impl_can_browse)	(RBSource *source);
-	const char *	(*impl_get_browser_key)	(RBSource *source);
+	char *		(*impl_get_browser_key)	(RBSource *source);
+	void		(*impl_browser_toggled)	(RBSource *source, gboolean enabled);
 
 	RBEntryView *	(*impl_get_entry_view)	(RBSource *source);
-	GList *		(*impl_get_extra_views)	(RBSource *source);
+	GList *		(*impl_get_property_views)	(RBSource *source);
 
-	GdkPixbuf *	(*impl_get_pixbuf)	(RBSource *source);
 	gboolean	(*impl_can_rename)	(RBSource *source);
 
 	gboolean	(*impl_can_search)	(RBSource *source);
@@ -92,8 +105,9 @@ struct _RBSourceClass
 	gboolean	(*impl_can_delete)	(RBSource *source);
 	gboolean	(*impl_can_move_to_trash) (RBSource *source);
 	gboolean	(*impl_can_copy)	(RBSource *source);
+	gboolean	(*impl_can_paste)	(RBSource *source);
 	gboolean	(*impl_can_add_to_queue)(RBSource *source);
-	
+
 	GList *		(*impl_cut)		(RBSource *source);
 	GList *		(*impl_copy)		(RBSource *source);
 	void		(*impl_paste)		(RBSource *source, GList *entries);
@@ -104,18 +118,21 @@ struct _RBSourceClass
 	void		(*impl_song_properties)	(RBSource *source);
 
 	gboolean	(*impl_try_playlist)	(RBSource *source);
+	guint		(*impl_want_uri)	(RBSource *source, const char *uri);
+	gboolean	(*impl_add_uri)		(RBSource *source, const char *uri, const char *title, const char *genre);
 
 	gboolean	(*impl_can_pause)	(RBSource *source);
 	RBSourceEOFType	(*impl_handle_eos)	(RBSource *source);
-	
+
 	gboolean	(*impl_have_url)	(RBSource *source);
 	gboolean	(*impl_receive_drag)	(RBSource *source, GtkSelectionData *data);
 	gboolean	(*impl_show_popup)	(RBSource *source);
-				   
+
 	void		(*impl_delete_thyself)	(RBSource *source);
 	void		(*impl_activate)	(RBSource *source);
 	void		(*impl_deactivate)	(RBSource *source);
-	gboolean	(*impl_disconnect)	(RBSource *source);
+	GList *		(*impl_get_ui_actions)	(RBSource *source);
+	GList *		(*impl_get_search_actions) (RBSource *source);
 };
 
 GType		rb_source_get_type		(void);
@@ -128,20 +145,20 @@ void		rb_source_update_play_statistics(RBSource *source, RhythmDB *db,
 						 RhythmDBEntry *entry);
 
 /* general interface */
-char *	        rb_source_get_status		(RBSource *source);
+void	        rb_source_set_pixbuf		(RBSource *source, GdkPixbuf *pixbuf);
+void	        rb_source_get_status		(RBSource *source, char **text, char **progress_text, float *progress);
 
 gboolean	rb_source_can_browse		(RBSource *source);
-const char *	rb_source_get_browser_key	(RBSource *source);
+char *		rb_source_get_browser_key	(RBSource *source);
+void		rb_source_browser_toggled	(RBSource *source, gboolean enabled);
 
 RBEntryView *	rb_source_get_entry_view	(RBSource *source);
 
-GList *		rb_source_get_extra_views	(RBSource *source);
+GList *		rb_source_get_property_views	(RBSource *source);
 
-GdkPixbuf *	rb_source_get_pixbuf		(RBSource *source);
 gboolean	rb_source_can_rename		(RBSource *source);
 
 gboolean	rb_source_can_search		(RBSource *source);
-
 void		rb_source_search		(RBSource *source,
 						 const char *text);
 
@@ -153,6 +170,7 @@ gboolean	rb_source_can_cut		(RBSource *source);
 gboolean	rb_source_can_delete		(RBSource *source);
 gboolean	rb_source_can_move_to_trash	(RBSource *source);
 gboolean	rb_source_can_copy		(RBSource *source);
+gboolean	rb_source_can_paste		(RBSource *source);
 gboolean	rb_source_can_add_to_queue	(RBSource *source);
 
 GList *		rb_source_cut			(RBSource *source);
@@ -165,12 +183,12 @@ void		rb_source_move_to_trash		(RBSource *source);
 void		rb_source_song_properties	(RBSource *source);
 
 gboolean	rb_source_try_playlist		(RBSource *source);
+guint		rb_source_want_uri		(RBSource *source, const char *uri);
+gboolean	rb_source_add_uri		(RBSource *source, const char *uri, const char *title, const char *genre);
 
 gboolean	rb_source_can_pause		(RBSource *source);
 RBSourceEOFType	rb_source_handle_eos		(RBSource *source);
 
-gboolean	rb_source_have_url		(RBSource *source);
-
 gboolean	rb_source_receive_drag		(RBSource *source, GtkSelectionData *data);
 
 gboolean	rb_source_show_popup		(RBSource *source);
@@ -179,11 +197,23 @@ void		rb_source_delete_thyself	(RBSource *source);
 
 void		rb_source_activate		(RBSource *source);
 void		rb_source_deactivate		(RBSource *source);
-gboolean	rb_source_disconnect		(RBSource *source);
 
-/* Protected method, should only be used by objects inheriting from RBSource */
-void            _rb_source_show_popup           (RBSource *source, 
+GList *		rb_source_get_ui_actions	(RBSource *source);
+GList *		rb_source_get_search_actions	(RBSource *source);
+
+GList *		rb_source_gather_selected_properties (RBSource *source, RhythmDBPropType prop);
+
+void            rb_source_set_hidden_when_empty (RBSource *source,
+                                                 gboolean  hidden);
+
+/* Protected methods, should only be used by objects inheriting from RBSource */
+void            _rb_source_show_popup           (RBSource *source,
 						 const char *ui_path);
+GtkActionGroup *_rb_source_register_action_group (RBSource *source,
+						  const char *group_name,
+						  GtkActionEntry *actions,
+						  int num_actions,
+						  gpointer user_data);
 
 G_END_DECLS
 
