@@ -85,6 +85,7 @@ struct RBHeaderPrivate
 
 	long elapsed_time;
 	long duration;
+	gboolean seekable;
 };
 
 enum
@@ -94,6 +95,7 @@ enum
 	PROP_SHELL_PLAYER,
 	PROP_ENTRY,
 	PROP_TITLE,
+	PROP_SEEKABLE,
 };
 
 #define SONG_MARKUP(xSONG) g_markup_printf_escaped ("<big><b>%s</b></big>", xSONG);
@@ -101,7 +103,6 @@ enum
 
 G_DEFINE_TYPE (RBHeader, rb_header, GTK_TYPE_HBOX)
 
-
 static void
 rb_header_class_init (RBHeaderClass *klass)
 {
@@ -122,10 +123,11 @@ rb_header_class_init (RBHeaderClass *klass)
 
 	g_object_class_install_property (object_class,
 					 PROP_ENTRY,
-					 g_param_spec_pointer ("entry",
-							       "RhythmDBEntry",
-							       "RhythmDBEntry pointer",
-							       G_PARAM_READWRITE));
+					 g_param_spec_boxed ("entry",
+							     "RhythmDBEntry",
+							     "RhythmDBEntry pointer",
+							     RHYTHMDB_TYPE_ENTRY,
+							     G_PARAM_READWRITE));
 	g_object_class_install_property (object_class,
 					 PROP_SHELL_PLAYER,
 					 g_param_spec_object ("shell-player",
@@ -141,6 +143,14 @@ rb_header_class_init (RBHeaderClass *klass)
 							      NULL,
 							      G_PARAM_READWRITE));
 
+	g_object_class_install_property (object_class,
+					 PROP_SEEKABLE,
+					 g_param_spec_boolean ("seekable",
+						 	       "seekable",
+							       "seekable",
+							       TRUE,
+							       G_PARAM_READWRITE));
+
 	g_type_class_add_private (klass, sizeof (RBHeaderPrivate));
 }
 
@@ -175,7 +185,7 @@ rb_header_init (RBHeader *header)
 
 	header->priv->song = gtk_label_new ("");
  	gtk_label_set_use_markup (GTK_LABEL (header->priv->song), TRUE);
- 	gtk_label_set_selectable (GTK_LABEL (header->priv->song), TRUE);	
+ 	gtk_label_set_selectable (GTK_LABEL (header->priv->song), TRUE);
 	gtk_label_set_ellipsize (GTK_LABEL (header->priv->song), PANGO_ELLIPSIZE_END);
 	gtk_box_pack_start (GTK_BOX (hbox), header->priv->song, TRUE, TRUE, 0);
 	gtk_widget_show (header->priv->song);
@@ -245,7 +255,7 @@ rb_header_set_property (GObject *object,
 		header->priv->db = g_value_get_object (value);
 		break;
 	case PROP_ENTRY:
-		header->priv->entry = g_value_get_pointer (value);
+		header->priv->entry = g_value_get_boxed (value);
 		if (header->priv->entry) {
 			header->priv->duration = rhythmdb_entry_get_ulong (header->priv->entry,
 									   RHYTHMDB_PROP_DURATION);
@@ -256,7 +266,7 @@ rb_header_set_property (GObject *object,
 	case PROP_SHELL_PLAYER:
 		header->priv->shell_player = g_value_get_object (value);
 		g_signal_connect (G_OBJECT (header->priv->shell_player),
-				  "elapsed-changed", 
+				  "elapsed-changed",
 				  (GCallback) rb_header_elapsed_changed_cb,
 				  header);
 		break;
@@ -264,13 +274,16 @@ rb_header_set_property (GObject *object,
 		g_free (header->priv->title);
 		header->priv->title = g_value_dup_string (value);
 		break;
+	case PROP_SEEKABLE:
+		header->priv->seekable = g_value_get_boolean (value);
+		break;
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
 		break;
 	}
 }
 
-static void 
+static void
 rb_header_get_property (GObject *object,
 			guint prop_id,
 			GValue *value,
@@ -283,7 +296,7 @@ rb_header_get_property (GObject *object,
 		g_value_set_object (value, header->priv->db);
 		break;
 	case PROP_ENTRY:
-		g_value_set_object (value, header->priv->entry);
+		g_value_set_boxed (value, header->priv->entry);
 		break;
 	case PROP_SHELL_PLAYER:
 		g_value_set_object (value, header->priv->shell_player);
@@ -291,6 +304,9 @@ rb_header_get_property (GObject *object,
 	case PROP_TITLE:
 		g_value_set_string (value, header->priv->title);
 		break;
+	case PROP_SEEKABLE:
+		g_value_set_boolean (value, header->priv->seekable);
+		break;
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
 		break;
@@ -302,7 +318,8 @@ rb_header_new (RBShellPlayer *shell_player)
 {
 	RBHeader *header;
 
-	header = RB_HEADER (g_object_new (RB_TYPE_HEADER, "shell-player", shell_player,
+	header = RB_HEADER (g_object_new (RB_TYPE_HEADER,
+					  "shell-player", shell_player,
 					  "title", NULL,
 					  "spacing", 6, NULL));
 
@@ -312,15 +329,20 @@ rb_header_new (RBShellPlayer *shell_player)
 }
 
 void
-rb_header_set_playing_entry (RBHeader *header, RhythmDBEntry *entry)
+rb_header_set_playing_entry (RBHeader *header,
+			     RhythmDBEntry *entry,
+			     gboolean seekable)
 {
-	g_object_set (G_OBJECT (header), "entry", entry, NULL);
+	g_object_set (header,
+		      "entry", entry,
+		      "seekable", seekable,
+		      NULL);
 }
 
 void
 rb_header_set_title (RBHeader *header, const char *title)
 {
-	g_object_set (G_OBJECT (header), "title", title, NULL);
+	g_object_set (header, "title", title, NULL);
 }
 
 void
@@ -329,11 +351,11 @@ rb_header_sync (RBHeader *header)
 	char *label_text;
 
 	rb_debug ("syncing with entry = %p", header->priv->entry);
-	
+
 	if (header->priv->entry != NULL) {
 		const char *song = header->priv->title;
 		const char *album;
-		const char *artist; 
+		const char *artist;
 
 		gboolean have_duration = (header->priv->duration > 0);
 
@@ -351,7 +373,7 @@ rb_header_sync (RBHeader *header)
 		gtk_label_set_markup (GTK_LABEL (header->priv->song), label_text);
 		g_free (label_text);
 
-		rb_header_set_show_timeline (header, have_duration);
+		rb_header_set_show_timeline (header, have_duration && header->priv->seekable);
 		if (have_duration)
 			rb_header_sync_time (header);
 	} else {
@@ -428,7 +450,7 @@ rb_header_sync_time (RBHeader *header)
 		header->priv->slider_locked = TRUE;
 		gtk_adjustment_set_value (header->priv->adjustment, progress);
 		header->priv->slider_locked = FALSE;
-		gtk_widget_set_sensitive (header->priv->scale, TRUE);
+		gtk_widget_set_sensitive (header->priv->scale, header->priv->seekable);
 	} else {
 		header->priv->slider_locked = TRUE;
 		gtk_adjustment_set_value (header->priv->adjustment, 0.0);
@@ -458,16 +480,16 @@ slider_moved_timeout (RBHeader *header)
 	long new;
 
 	GDK_THREADS_ENTER ();
-	
+
 	progress = gtk_adjustment_get_value (gtk_range_get_adjustment (GTK_RANGE (header->priv->scale)));
 	new = (long) (progress+0.5);
-	
+
 	rb_debug ("setting time to %ld", new);
 	rb_shell_player_set_playing_time (header->priv->shell_player, new, NULL);
 
 	header->priv->latest_set_time = new;
 	header->priv->slider_moved_timeout = 0;
-	
+
 	GDK_THREADS_LEAVE ();
 
 	return FALSE;
@@ -500,7 +522,7 @@ slider_moved_callback (GtkWidget *widget,
 	}
 	header->priv->slider_moved_timeout =
 		g_timeout_add (40, (GSourceFunc) slider_moved_timeout, header);
-	
+
 	return FALSE;
 }
 
@@ -548,7 +570,7 @@ changed_idle_callback (RBHeader *header)
 	slider_release_callback (header->priv->scale, NULL, header);
 
 	header->priv->value_changed_update_handler = 0;
-	rb_debug ("in changed_idle_callback"); 
+	rb_debug ("in changed_idle_callback");
 
 	GDK_THREADS_LEAVE ();
 
@@ -585,8 +607,7 @@ rb_header_update_elapsed (RBHeader *header)
 	g_free (elapsed_text);
 }
 
-
-static void 
+static void
 rb_header_elapsed_changed_cb (RBShellPlayer *player,
 			      guint elapsed,
 			      RBHeader *header)
@@ -594,4 +615,3 @@ rb_header_elapsed_changed_cb (RBShellPlayer *player,
 	header->priv->elapsed_time = elapsed;
 	rb_header_sync_time (header);
 }
-
