@@ -64,8 +64,6 @@ static void rb_library_browser_views_changed (GConfClient *client,
 					      GConfEntry *entry,
 					      RBLibraryBrowser *widget);
 
-
-
 G_DEFINE_TYPE (RBLibraryBrowser, rb_library_browser, GTK_TYPE_HBOX)
 #define RB_LIBRARY_BROWSER_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), RB_TYPE_LIBRARY_BROWSER, RBLibraryBrowserPrivate))
 
@@ -83,7 +81,6 @@ typedef struct
 	GHashTable *selections;
 } RBLibraryBrowserPrivate;
 
-
 enum
 {
 	PROP_0,
@@ -138,10 +135,11 @@ rb_library_browser_class_init (RBLibraryBrowserClass *klass)
 							      G_PARAM_READABLE));
 	g_object_class_install_property (object_class,
 					 PROP_ENTRY_TYPE,
-					 g_param_spec_pointer ("entry-type",
-						 	       "Entry type",
-							       "Type of entry to display in this browser",
-							       G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
+					 g_param_spec_boxed ("entry-type",
+						 	     "Entry type",
+							     "Type of entry to display in this browser",
+							     RHYTHMDB_TYPE_ENTRY_TYPE,
+							     G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
 
 	g_type_class_add_private (klass, sizeof (RBLibraryBrowserPrivate));
 }
@@ -158,7 +156,8 @@ rb_library_browser_init (RBLibraryBrowser *widget)
 }
 
 static GObject *
-rb_library_browser_constructor (GType type, guint n_construct_properties,
+rb_library_browser_constructor (GType type,
+				guint n_construct_properties,
 				GObjectConstructParam *construct_properties)
 {
 	RBLibraryBrowserClass *klass;
@@ -179,7 +178,7 @@ rb_library_browser_constructor (GType type, guint n_construct_properties,
 					     browser_properties[i].type,
 					     _(browser_properties[i].name));
 		g_hash_table_insert (priv->property_views, (gpointer)(browser_properties[i].type), view);
-	
+
 		rb_property_view_set_selection_mode (view, GTK_SELECTION_MULTIPLE);
 		g_signal_connect_object (G_OBJECT (view),
 					 "properties-selected",
@@ -191,7 +190,7 @@ rb_library_browser_constructor (GType type, guint n_construct_properties,
 					 browser, 0);
 		gtk_widget_show_all (GTK_WIDGET (view));
 		gtk_widget_set_no_show_all (GTK_WIDGET (view), TRUE);
-		gtk_box_pack_start_defaults (GTK_BOX (browser), GTK_WIDGET (view));	     
+		gtk_box_pack_start_defaults (GTK_BOX (browser), GTK_WIDGET (view));
 	}
 
 	update_browser_views_visibility (browser);
@@ -208,7 +207,15 @@ rb_library_browser_finalize (GObject *object)
 
 	eel_gconf_notification_remove (priv->browser_view_notify_id);
 
-	g_object_unref (G_OBJECT (priv->db));
+	if (priv->db != NULL) {
+		g_object_unref (priv->db);
+	}
+	if (priv->input_model != NULL) {
+		g_object_unref (priv->input_model);
+	}
+	if (priv->output_model != NULL) {
+		g_object_unref (priv->output_model);
+	}
 
 	g_hash_table_destroy (priv->property_views);
 	g_hash_table_destroy (priv->selections);
@@ -224,19 +231,19 @@ rb_library_browser_set_property (GObject *object,
 {
 	RBLibraryBrowserPrivate *priv = RB_LIBRARY_BROWSER_GET_PRIVATE (object);
 
-	switch (prop_id)
-	{
+	switch (prop_id) {
 	case PROP_DB:
-		if (priv->db) {
-			g_object_unref (G_OBJECT (priv->db));
+		if (priv->db != NULL) {
+			g_object_unref (priv->db);
 		}
 		priv->db = g_value_get_object (value);
 
-		if (priv->db)
+		if (priv->db != NULL) {
 			g_object_ref (priv->db);
+		}
 		break;
 	case PROP_ENTRY_TYPE:
-		priv->entry_type = g_value_get_pointer (value);
+		priv->entry_type = g_value_get_boxed (value);
 		break;
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
@@ -252,8 +259,7 @@ rb_library_browser_get_property (GObject *object,
 {
 	RBLibraryBrowserPrivate *priv = RB_LIBRARY_BROWSER_GET_PRIVATE (object);
 
-	switch (prop_id)
-	{
+	switch (prop_id) {
 	case PROP_DB:
 		g_value_set_object (value, priv->db);
 		break;
@@ -264,7 +270,7 @@ rb_library_browser_get_property (GObject *object,
 		g_value_set_object (value, priv->output_model);
 		break;
 	case PROP_ENTRY_TYPE:
-		g_value_set_pointer (value, priv->entry_type);
+		g_value_set_boxed (value, priv->entry_type);
 		break;
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
@@ -273,7 +279,8 @@ rb_library_browser_get_property (GObject *object,
 }
 
 RBLibraryBrowser *
-rb_library_browser_new (RhythmDB *db, RhythmDBEntryType entry_type)
+rb_library_browser_new (RhythmDB *db,
+			RhythmDBEntryType entry_type)
 {
 	RBLibraryBrowser *widget;
 
@@ -286,10 +293,12 @@ rb_library_browser_new (RhythmDB *db, RhythmDBEntryType entry_type)
 }
 
 static void
-update_browser_property_visibilty (RhythmDBPropType prop, RBPropertyView *view, GList *properties)
+update_browser_property_visibilty (RhythmDBPropType prop,
+				   RBPropertyView *view,
+				   GList *properties)
 {
 	gboolean old_vis, new_vis;
-	
+
 	old_vis = GTK_WIDGET_VISIBLE (view);
 	new_vis = (g_list_find (properties, (gpointer)prop) != NULL);
 
@@ -342,7 +351,8 @@ view_property_selected_cb (RBPropertyView *view,
 	rb_library_browser_set_selection (widget, prop, selection);
 }
 static void
-view_selection_reset_cb (RBPropertyView *view, RBLibraryBrowser *widget)
+view_selection_reset_cb (RBPropertyView *view,
+			 RBLibraryBrowser *widget)
 {
 	RhythmDBPropType prop;
 
@@ -351,7 +361,9 @@ view_selection_reset_cb (RBPropertyView *view, RBLibraryBrowser *widget)
 }
 
 static void
-reset_view_cb (RhythmDBPropType prop, RBPropertyView *view, RBLibraryBrowser *widget)
+reset_view_cb (RhythmDBPropType prop,
+	       RBPropertyView *view,
+	       RBLibraryBrowser *widget)
 {
 	rb_property_view_set_selection (view, NULL);
 }
@@ -376,7 +388,9 @@ typedef struct {
 } ConstructQueryData;
 
 static void
-construct_query_cb (RhythmDBPropType type, GList *selections, ConstructQueryData *data)
+construct_query_cb (RhythmDBPropType type,
+		    GList *selections,
+		    ConstructQueryData *data)
 {
 	rhythmdb_query_append_prop_multiple (data->db,
 					     data->query,
@@ -384,13 +398,13 @@ construct_query_cb (RhythmDBPropType type, GList *selections, ConstructQueryData
 					     selections);
 }
 
-RhythmDBQuery*
+RhythmDBQuery *
 rb_library_browser_construct_query (RBLibraryBrowser *widget)
 {
 	RBLibraryBrowserPrivate *priv = RB_LIBRARY_BROWSER_GET_PRIVATE (widget);
 	RhythmDBQuery *query;
 	ConstructQueryData *data;
-	
+
 	query = g_ptr_array_new ();
 	data = g_new0 (ConstructQueryData, 1);
 	data->widget = widget;
@@ -424,7 +438,9 @@ prop_to_index (RhythmDBPropType type)
 }
 
 static void
-ignore_selection_changes (RBLibraryBrowser *widget, RBPropertyView *view, gboolean block)
+ignore_selection_changes (RBLibraryBrowser *widget,
+			  RBPropertyView *view,
+			  gboolean block)
 {
 	if (block) {
 		g_signal_handlers_block_by_func (view, view_selection_reset_cb, widget);
@@ -474,7 +490,7 @@ restore_selection (RBLibraryBrowser *widget,
 	selections = g_hash_table_lookup (priv->selections, (gpointer)browser_properties[property_index].type);
 
 	if (query_pending) {
-		g_object_ref (G_OBJECT (widget));
+		g_object_ref (widget);
 
 		data = g_new0 (SelectionRestoreData, 1);
 		data->widget = widget;
@@ -482,7 +498,7 @@ restore_selection (RBLibraryBrowser *widget,
 		data->selections = selections;
 		data->model = priv->input_model;
 
-		data->handler_id = 
+		data->handler_id =
 			g_signal_connect_data (priv->input_model,
 					       "complete",
 					       G_CALLBACK (query_complete_cb),
@@ -496,7 +512,9 @@ restore_selection (RBLibraryBrowser *widget,
 }
 
 static void
-rebuild_child_model (RBLibraryBrowser *widget, gint property_index, gboolean query_pending)
+rebuild_child_model (RBLibraryBrowser *widget,
+		     gint property_index,
+		     gboolean query_pending)
 {
 	RBLibraryBrowserPrivate *priv = RB_LIBRARY_BROWSER_GET_PRIVATE (widget);
 	RhythmDBPropertyModel *prop_model;
@@ -511,18 +529,17 @@ rebuild_child_model (RBLibraryBrowser *widget, gint property_index, gboolean que
 	/* get the query model for the previous property view */
 	view = g_hash_table_lookup (priv->property_views, (gpointer)browser_properties[property_index].type);
 	prop_model = rb_property_view_get_model (view);
-	g_object_get (G_OBJECT (prop_model), "query-model", &base_model, NULL);
-	g_object_unref (G_OBJECT (prop_model));
+	g_object_get (prop_model, "query-model", &base_model, NULL);
 
 	selections = g_hash_table_lookup (priv->selections, (gpointer)browser_properties[property_index].type);
 	if (selections != NULL) {
 
-		/* create a new query model based on it, filtered by 
+		/* create a new query model based on it, filtered by
 		 * the selections of the previous property view.
 		 * we need the entry type query criteria to allow the
 		 * backend to optimise the query.
 		 */
-		query = rhythmdb_query_parse (priv->db, 
+		query = rhythmdb_query_parse (priv->db,
 				              RHYTHMDB_QUERY_PROP_EQUALS, RHYTHMDB_PROP_TYPE, priv->entry_type,
 					      RHYTHMDB_QUERY_END);
 		rhythmdb_query_append_prop_multiple (priv->db,
@@ -533,7 +550,7 @@ rebuild_child_model (RBLibraryBrowser *widget, gint property_index, gboolean que
 		child_model = rhythmdb_query_model_new_empty (priv->db);
 		if (query_pending) {
 			rb_debug ("rebuilding child model for browser %d; query is pending", property_index);
-			g_object_set (G_OBJECT (child_model), 
+			g_object_set (child_model,
 				      "query", query,
 				      "base-model", base_model,
 				      NULL);
@@ -547,18 +564,20 @@ rebuild_child_model (RBLibraryBrowser *widget, gint property_index, gboolean que
 		rhythmdb_query_free (query);
 	} else {
 		rb_debug ("no selection for browser %d - reusing parent model", property_index);
-		child_model = base_model;
+		child_model = g_object_ref (base_model);
 	}
-	
+
 	/* If this is the last property, use the child model as the output model
 	 * for the browser.  Otherwise, use it as the input for the next property
 	 * view.
 	 */
 	if (property_index == num_browser_properties-1) {
-		if (priv->output_model)
-			g_object_unref (G_OBJECT (priv->output_model));
+		if (priv->output_model != NULL) {
+			g_object_unref (priv->output_model);
+		}
 
 		priv->output_model = child_model;
+
 		g_object_notify (G_OBJECT (widget), "output-model");
 
 	} else {
@@ -566,26 +585,26 @@ rebuild_child_model (RBLibraryBrowser *widget, gint property_index, gboolean que
 		ignore_selection_changes (widget, view, TRUE);
 
 		prop_model = rb_property_view_get_model (view);
-		g_object_set (G_OBJECT (prop_model), "query-model", child_model, NULL);
-		g_object_unref (G_OBJECT (prop_model));
+		g_object_set (prop_model, "query-model", child_model, NULL);
 
-		g_object_unref (G_OBJECT (base_model));
+		g_object_unref (child_model);
 
 		rebuild_child_model (widget, property_index + 1, query_pending);
 		restore_selection (widget, property_index + 1, query_pending);
 	}
 
-	if (child_model != base_model)
-		g_object_unref (G_OBJECT (child_model));
+	g_object_unref (base_model);
 }
 
 void
-rb_library_browser_set_selection (RBLibraryBrowser *widget, RhythmDBPropType type, GList *selection)
+rb_library_browser_set_selection (RBLibraryBrowser *widget,
+				  RhythmDBPropType type,
+				  GList *selection)
 {
 	RBLibraryBrowserPrivate *priv = RB_LIBRARY_BROWSER_GET_PRIVATE (widget);
 	GList *old_selection;
 	RBPropertyView *view;
-	
+
 	old_selection = g_hash_table_lookup (priv->selections, (gpointer)type);
 
 	if (rb_string_list_equal (old_selection, selection))
@@ -613,7 +632,7 @@ rb_library_browser_get_property_views (RBLibraryBrowser *widget)
 	return rb_collate_hash_table_values (priv->property_views);
 }
 
-RBPropertyView*
+RBPropertyView *
 rb_library_browser_get_property_view (RBLibraryBrowser *widget,
 				      RhythmDBPropType type)
 {
@@ -625,7 +644,7 @@ rb_library_browser_get_property_view (RBLibraryBrowser *widget,
 }
 
 void
-rb_library_browser_set_model (RBLibraryBrowser *widget, 
+rb_library_browser_set_model (RBLibraryBrowser *widget,
 			      RhythmDBQueryModel *model,
 			      gboolean query_pending)
 {
@@ -633,20 +652,22 @@ rb_library_browser_set_model (RBLibraryBrowser *widget,
 	RBPropertyView *view;
 	RhythmDBPropertyModel *prop_model;
 
-	if (priv->input_model)
-		g_object_unref (G_OBJECT (priv->input_model));
+	if (priv->input_model != NULL) {
+		g_object_unref (priv->input_model);
+	}
 
 	priv->input_model = model;
-	g_object_ref (G_OBJECT (model));
+
+	if (priv->input_model != NULL) {
+		g_object_ref (priv->input_model);
+	}
 
 	view = g_hash_table_lookup (priv->property_views, (gpointer)browser_properties[0].type);
 	ignore_selection_changes (widget, view, TRUE);
 
 	prop_model = rb_property_view_get_model (view);
-	g_object_set (G_OBJECT (prop_model), "query-model", priv->input_model, NULL);
-	g_object_unref (G_OBJECT (prop_model));
+	g_object_set (prop_model, "query-model", priv->input_model, NULL);
 
 	rebuild_child_model (widget, 0, query_pending);
 	restore_selection (widget, 0, query_pending);
 }
-
